# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils module for Neural Insights server."""
import os
import socket
from importlib.util import find_spec
from pathlib import Path
from typing import Optional, Union

from neural_insights.utils.exceptions import ClientErrorException, NotFoundException


def determine_ip() -> str:
    """Return IP to be used by server."""
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        sock.connect(("10.0.0.0", 1))
        ip = sock.getsockname()[0]
    except Exception:
        ip = "127.0.0.1"
    finally:
        sock.close()

    return ip


def get_size(path: str, unit: str = "MB", add_unit: bool = False) -> Union[str, int]:
    """Check file or directory size."""
    supported_units = {
        "B": 1,
        "KB": 1024,
        "MB": 1024**2,
        "GB": 1024**3,
    }
    unit_modifier = supported_units.get(unit, None)
    if unit_modifier is None:
        raise Exception(
            "Unit not supported. Select one of following: " + str(supported_units.keys()),
        )
    root_dir = Path(path)
    if root_dir.is_file():
        size = root_dir.stat().st_size
    else:
        size = sum(f.stat().st_size for f in root_dir.glob("**/*") if f.is_file())
    size = int(round(size / unit_modifier))
    if add_unit:
        return f"{size}{unit}"
    return size


def check_module(module_name: str) -> None:
    """Check if module exists.

    Raise exception when not found.
    """
    if module_name == "onnxrt":
        module_name = "onnxruntime"
    if module_name == "pytorch":
        module_name = "torch"
    module = find_spec(module_name.lower())
    if module is None:
        raise ClientErrorException(f"Could not find {module_name} module.")


def get_file_extension(path: str) -> str:
    """Get file extension without leading dot."""
    return os.path.splitext(path)[1][1:]


def get_framework_from_path(model_path: str) -> Optional[str]:
    """Get framework name from model extension.

    :param model_path: Path to model.
    """
    from neural_insights.components.model.repository import ModelRepository

    model_repository = ModelRepository()
    try:
        model = model_repository.get_model(model_path)
        return model.get_framework_name()
    except NotFoundException:
        return None



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The OpEntry class."""
from typing import Any, Dict

from neural_insights.utils.json_serializer import JsonSerializer


class OpEntry(JsonSerializer):
    """OP entry class."""

    def __init__(self, op_name: str, mse: float, activation_min: float, activation_max: float):
        """Initialize OP entry."""
        super().__init__()
        self.op_name: str = op_name
        self.mse: float = mse
        self.activation_min: float = activation_min
        self.activation_max: float = activation_max

    def serialize(
        self,
        serialization_type: str = "default",
    ) -> Dict[str, Any]:
        """Serialize OP entry."""
        return {
            "OP name": self.op_name,
            "MSE": self.mse,
            "Activation Min": self.activation_min,
            "Activation Max": self.activation_max,
        }



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Data shape class."""
from typing import Any, Dict, List, Optional, Union

from neural_insights.utils.json_serializer import JsonSerializer


class Shape(JsonSerializer):
    """Data shape definition."""

    def __init__(self, shape: Optional[str] = "", trusted: bool = False) -> None:
        """Object construction."""
        super().__init__()

        self.shape = shape
        self.trusted = trusted

    def serialize(
        self,
        serialization_type: str = "default",
    ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
        """Serialize Shape class to dict."""
        result = {}
        for key, value in self.__dict__.items():
            if key in self._skip:
                continue
            result.update({key: value})
        return result



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Onnxrt model class."""
import os
import re
import sys
from typing import Any, List, Optional

from neural_compressor.experimental.common.model import Model as NCModel
from neural_compressor.model.onnx_model import ONNXModel
from neural_compressor.utils.logger import Logger
from neural_insights.components.graph.graph import Graph
from neural_insights.components.graph.reader.onnxrt_reader import OnnxrtReader
from neural_insights.components.model.domain import Domain
from neural_insights.components.model.model import Model
from neural_insights.components.model.shape import Shape
from neural_insights.utils.consts import DomainFlavours, Domains, Frameworks
from neural_insights.utils.logger import log
from neural_insights.utils.utils import check_module, get_file_extension


class OnnxrtModel(Model):
    """Onnxrt Model class."""

    def __init__(self, path: str) -> None:
        """Initialize object."""
        super().__init__(path)
        self._nc_model_instance: Optional[ONNXModel] = None

    @property
    def domain(self) -> Domain:
        """Get model domain."""
        try:
            input_node_names = {node.name for node in self.nc_model_instance.graph().input}  # pylint: disable=E1101
            node_names = {node.name for node in self.nc_model_instance.nodes()}  # pylint: disable=E1101
            boundary_nodes = [node.name for node in self.nc_model_instance.graph().input]  # pylint: disable=E1101
            boundary_nodes.extend(
                [node.name for node in self.nc_model_instance.graph().output],  # pylint: disable=E1101
            )
            op_names = {node.op_type for node in self.nc_model_instance.nodes()}  # pylint: disable=E1101

        except Exception:
            return Domain()

        nlp_domain = self._check_nlp_domain(input_node_names)
        if nlp_domain is not None:
            return nlp_domain

        object_detection_domain = self._check_object_detection_domain(
            node_names,
            boundary_nodes,
            op_names,
        )
        if object_detection_domain is not None:
            return object_detection_domain

        image_recognition_domain = self._check_image_recognition_domain(op_names)
        if image_recognition_domain is not None:
            return image_recognition_domain

        return Domain()

    @staticmethod
    def _has_any_name_parts(nodes: set, name_parts: List[str]) -> bool:
        """Check if there is at least one node for name_parts."""
        matching_names = []
        for node in nodes:
            for partial_name in name_parts:
                search = re.match(partial_name, node)
                if search:
                    matching_names.append(node)
        return bool(matching_names)

    def _check_nlp_domain(self, input_node_names: set) -> Optional[Domain]:
        """Check if model fits into NLP domain."""
        if self._has_any_name_parts(
            input_node_names,
            ["input_ids", "input_mask", "segment_ids"],
        ):
            return Domain(
                domain=Domains.NLP.value,
                domain_flavour=DomainFlavours.NONE.value,
            )
        return None

    def _check_object_detection_domain(
        self,
        node_names: set,
        boundary_nodes: list,
        op_names: set,
    ) -> Optional[Domain]:
        """Check if model fits into Object Detection domain."""
        if (
            self._has_any_name_parts(
                node_names,
                ["NonMaxSuppression", "box", "score", "class", "detection"],
            )
            or self._has_any_name_parts(op_names, ["NonMaxSuppression"])
            or self._has_any_name_parts(
                set(boundary_nodes),
                ["NonMaxSuppression", "box", "score", "class", "detection"],
            )
        ):
            return Domain(
                domain=Domains.OBJECT_DETECTION.value,
                domain_flavour=DomainFlavours.NONE.value,
            )
        return None

    def _check_image_recognition_domain(self, op_names: set) -> Optional[Domain]:
        """Check if model fits into Image Recognition domain."""
        if self._has_any_name_parts(op_names, ["Conv", "Relu/Clip"]):
            return Domain(
                domain=Domains.IMAGE_RECOGNITION.value,
                domain_flavour=DomainFlavours.NONE.value,
            )
        return None

    @property
    def input_shape(self) -> Shape:
        """Try to detect data shape."""
        from google.protobuf.json_format import MessageToDict

        shape = None
        trusted = False

        if self.domain.domain == Domains.IMAGE_RECOGNITION.value:
            shape_list = []

            for input_node in self.filtered_input_nodes:
                node_dict = MessageToDict(input_node)
                dimensions = node_dict.get("type", {}).get("tensorType", {}).get("shape", {}).get("dim", [])
                input_shape = []
                for dim in dimensions:
                    if dim.get("dimValue", None) is not None:
                        input_shape.append(dim["dimValue"])
                shape_list.append(",".join(input_shape))

            if len(shape_list) == 1:
                shape = shape_list[0]
                shape = remove_number_of_samples_from_shape(shape)
                trusted = True
            if len(shape_list) > 1:
                adjusted_shapes_list = []
                for sub_shape in shape_list:
                    adjusted_shapes_list.append(
                        remove_number_of_samples_from_shape(sub_shape),
                    )
                shape = ",".join(
                    ["[" + sub_shape + "]" for sub_shape in adjusted_shapes_list],
                )
                trusted = True

        return Shape(shape=shape, trusted=trusted)

    @property
    def shape_elements_order(self) -> List[str]:
        """Get order of input shape channels."""
        return ["channels", "height", "width"]

    @staticmethod
    def get_framework_name() -> str:
        """Get the name of framework."""
        return Frameworks.ONNX.value

    def get_model_graph(self) -> Graph:
        """Get model Graph."""
        graph_reader = OnnxrtReader(self)
        return graph_reader.read()

    @property
    def nc_model_instance(self) -> ONNXModel:
        """Get Neural Compressor Model instance."""
        self._ensure_nc_model_instance()
        return self._nc_model_instance

    def _ensure_nc_model_instance(self) -> None:
        """Create Neural Compressor Model instance if needed."""
        if self._nc_model_instance is not None:
            return
        model_name = os.path.splitext(os.path.basename(self.path))[0]
        Logger().get_logger().setLevel(log.level)
        self._nc_model_instance = NCModel(self.path)
        self._nc_model_instance.name = model_name

    @staticmethod
    def supports_path(path: str) -> bool:
        """Check if given path is of supported model."""
        return "onnx" == get_file_extension(path)

    def guard_requirements_installed(self) -> None:
        """Ensure all requirements are installed."""
        check_module("onnx")
        check_module("onnxruntime")
        if sys.version_info < (3, 11):  # pragma: no cover
            check_module("onnxruntime_extensions")

    @property
    def filtered_input_nodes(self) -> List[Any]:
        """Get filtered input nodes."""
        input_nodes = self.nc_model_instance.graph().input  # pylint: disable=E1101
        name_to_input = {}
        for input in input_nodes:
            name_to_input[input.name] = input
        for initializer in self.nc_model_instance.graph().initializer:  # pylint: disable=E1101
            if initializer.name in name_to_input:
                input_nodes.remove(name_to_input[initializer.name])
        return input_nodes


def remove_number_of_samples_from_shape(shape: str) -> str:
    """Remove number of samples from shape if exists."""
    shape_as_list = shape.split(",")
    if len(shape_as_list) > 3:
        shape_as_list.pop(0)
    return ",".join(shape_as_list)



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test saved_model Model."""

import unittest
from unittest.mock import MagicMock, patch

from neural_insights.components.model.tensorflow.saved_model import SavedModelModel
from neural_insights.utils.consts import Frameworks


class TestSavedModelModel(unittest.TestCase):
    """Test SavedModelModel class."""

    def setUp(self) -> None:
        """Prepare environment."""
        super().setUp()

        get_model_type_patcher = patch(
            "neural_insights.components.model.model_type_getter.nc_get_model_type",
        )
        self.addCleanup(get_model_type_patcher.stop)
        get_model_type_mock = get_model_type_patcher.start()
        get_model_type_mock.side_effect = self._get_model_type

        nc_tensorflow_model_patcher = patch(
            "neural_insights.components.model.tensorflow.model.NCModel",
        )
        self.addCleanup(nc_tensorflow_model_patcher.stop)
        nc_model_instance_mock = nc_tensorflow_model_patcher.start()
        nc_model_instance_mock.return_value.input_node_names = [
            "first input node",
            "second input node",
        ]
        nc_model_instance_mock.return_value.output_node_names = [
            "first output node",
            "second output node",
        ]

    def _get_model_type(self, path: str) -> str:
        """Return model type for well known paths."""
        if "/path/to/saved_model" == path:
            return "saved_model"
        raise ValueError()

    def test_get_framework_name(self) -> None:
        """Test getting correct framework name."""
        self.assertEqual(Frameworks.TF.value, SavedModelModel.get_framework_name())

    def test_supports_correct_path(self) -> None:
        """Test getting correct framework name."""
        self.assertTrue(SavedModelModel.supports_path("/path/to/saved_model"))

    def test_supports_incorrect_path(self) -> None:
        """Test getting correct framework name."""
        self.assertFalse(SavedModelModel.supports_path("/path/to/model.txt"))

    @patch("neural_insights.components.model.tensorflow.model.check_module")
    def test_guard_requirements_installed(self, mocked_check_module: MagicMock) -> None:
        """Test guard_requirements_installed."""
        model = SavedModelModel("/path/to/saved_model")

        model.guard_requirements_installed()

        mocked_check_module.assert_called_once_with("tensorflow")

    def test_get_input_nodes(self) -> None:
        """Test getting input nodes."""
        model = SavedModelModel("/path/to/saved_model")
        self.assertEqual(["first input node", "second input node"], model.get_input_nodes())

    def test_get_output_nodes(self) -> None:
        """Test getting output nodes."""
        model = SavedModelModel("/path/to/saved_model")
        self.assertEqual(
            ["first output node", "second output node", "custom"],
            model.get_output_nodes(),
        )

    def test_get_input_and_output_nodes(self) -> None:
        """Test getting input nodes."""
        model = SavedModelModel("/path/to/saved_model")
        self.assertEqual(["first input node", "second input node"], model.get_input_nodes())
        self.assertEqual(
            ["first output node", "second output node", "custom"],
            model.get_output_nodes(),
        )

    def test_shape_elements_order(self) -> None:
        """Test getting shape elements order."""
        model = SavedModelModel("/path/to/saved_model")
        self.assertListEqual(model.shape_elements_order, ["height", "width", "channels"])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Json Serializer test."""

import unittest
from enum import Enum
from typing import Any, List

from neural_insights.utils.json_serializer import JsonSerializer


class SubClassToTest(JsonSerializer):
    """Test sub class for json serializer tests."""

    def __init__(self) -> None:
        """Initialize test class."""
        super().__init__()
        self.some_variable: str = "value"


class SomEnum(Enum):
    """Enum to test."""

    FOO = "foo"
    BAR = 12


class ClassToTest(JsonSerializer):
    """Test class for json serializer tests."""

    def __init__(self) -> None:
        """Initialize test class."""
        super().__init__()
        self._skip.append("skip_test")
        self.skip_test: str = "this should be skipped"
        self.none_value: Any = None
        self.string: str = "some string"
        self.empty_list: List = []
        self.some_list: List[Any] = ["a", 1]
        self._private_var: int = 1
        self.sub = SubClassToTest()
        subclass1 = SubClassToTest()
        subclass1.some_variable = "this is subclass1 variable"
        subclass2 = SubClassToTest()
        subclass2.some_variable = "this is subclass2 variable"
        self.sub_list: List[SubClassToTest] = [subclass1, subclass2]
        self.empty_dict: dict = {}
        self.dict_with_empty_values = {
            "foo": None,
            "bar": None,
        }
        self.dict_with_some_empty_values = {
            "foo": None,
            "bar": 12,
            "baz": None,
            "donk": 42,
        }
        self.enum_value = SomEnum.FOO


class TestJsonSerializer(unittest.TestCase):
    """Json serializer tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Parser tests constructor."""
        super().__init__(*args, **kwargs)

    def test_serialization(self) -> None:
        """Test if path is correctly recognized as hidden."""
        test_object = ClassToTest()
        result = test_object.serialize()
        expected = {
            "string": "some string",
            "some_list": ["a", 1],
            "private_var": 1,
            "sub": {
                "some_variable": "value",
            },
            "sub_list": [
                {
                    "some_variable": "this is subclass1 variable",
                },
                {
                    "some_variable": "this is subclass2 variable",
                },
            ],
            "dict_with_some_empty_values": {
                "bar": 12,
                "donk": 42,
            },
            "enum_value": "foo",
        }
        self.assertEqual(type(result), dict)
        self.assertDictEqual(result, expected)  # type: ignore


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Graph collapser."""

from typing import Dict, List, Set

from neural_insights.components.graph.graph import Graph
from neural_insights.components.graph.node import GroupNode, Node


class Collapser:
    """Graph collapser."""

    GROUP_NAME_PREFIX = "node_group_"

    def __init__(self, expanded_groups: List[str] = []) -> None:
        """Construct the collapser."""
        restored_group_names = [self._unprepare_group_name(name) for name in expanded_groups]
        self.expanded_groups: Set[str] = set(restored_group_names)
        self.group_sizes: Dict[str, int] = {}

    def collapse(self, graph: Graph) -> Graph:
        """Return graph with collapsed nodes."""
        collapsed_graph = Graph()

        self._calculate_group_sizes(graph)
        self._add_nodes_to_collapsed_graph(collapsed_graph, graph)
        self._add_edges_to_collapsed_graph(collapsed_graph, graph)

        return collapsed_graph

    def _calculate_group_sizes(self, graph: Graph) -> None:
        """Create dict with number of nodes in each group."""
        self.group_sizes = {}
        for node in graph.nodes:
            for group in node.groups:
                if not self.group_sizes.get(group):
                    self.group_sizes[group] = 0
                self.group_sizes[group] += 1

    def _add_nodes_to_collapsed_graph(self, collapsed_graph: Graph, graph: Graph) -> None:
        """Add Nodes from source graph to collapsed graph."""
        for node in graph.nodes:
            collapsed_graph.add_node(self._get_node_for_collapsed_graph(node))

    def _add_edges_to_collapsed_graph(self, collapsed_graph: Graph, graph: Graph) -> None:
        """Add Edges from source graph to collapsed graph."""
        collapsed_edges_repository: Dict[str, bool] = {}

        for edge in graph.edges:
            edge_added = collapsed_graph.add_edge(
                source_id=edge.source,
                target_id=edge.target,
            )
            if edge_added:
                continue

            source_node_id = self._get_collapsed_node_id(graph.get_node(edge.source))
            target_node_id = self._get_collapsed_node_id(graph.get_node(edge.target))

            if source_node_id == target_node_id:
                # skip edges inside collapsed node
                continue

            collapsed_edge_designation = f"collapsed edge between {source_node_id} and {target_node_id}"
            is_collapsed_already_added = collapsed_edges_repository.get(
                collapsed_edge_designation,
                False,
            )
            if not is_collapsed_already_added:
                collapsed_graph.add_edge(
                    source_id=source_node_id,
                    target_id=target_node_id,
                )
                collapsed_edges_repository[collapsed_edge_designation] = True

    def _get_node_for_collapsed_graph(self, node: Node) -> Node:
        """Return a node for collapsed graph."""
        if self._should_collapse_node(node):
            return self._collapse_node(node)
        return node

    def _should_collapse_node(self, node: Node) -> bool:
        """Decide if given node should be collapsed."""
        if not node.groups:
            return False

        for group in node.groups:
            if group not in self.expanded_groups:
                return self.group_sizes.get(group, 0) > 1

        return False

    def _collapse_node(self, node: Node) -> Node:
        """Collapse node into group node."""
        collapsed_node_id = self._get_collapsed_node_id(node)
        group_name = self._unprepare_group_name(collapsed_node_id)
        return GroupNode(id=collapsed_node_id, group_name=group_name)

    def _get_collapsed_node_id(self, node: Node) -> str:
        """Collapse node into group node."""
        if not self._should_collapse_node(node):
            return node.id

        for group in node.groups:
            if group not in self.expanded_groups:
                break

        return self._prepare_group_name(group)

    def _prepare_group_name(self, name: str) -> str:
        """Convert group name to unique one."""
        return f"{self.GROUP_NAME_PREFIX}{name}"

    def _unprepare_group_name(self, name: str) -> str:
        """Convert group name to unique one."""
        if not name.startswith(self.GROUP_NAME_PREFIX):
            return name

        return name[len(self.GROUP_NAME_PREFIX) :]



# -*- coding: utf-8 -*-
#
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Neural Insights main class."""
from os import PathLike
from typing import Optional

from neural_insights.components.workload_manager.quantization_workload import QuantizationWorkload
from neural_insights.components.workload_manager.workload import Workload
from neural_insights.components.workload_manager.workload_manager import WorkloadManager
from neural_insights.utils.consts import WorkloadModes


class NeuralInsights:
    def __init__(self, workdir_location: PathLike) -> None:
        self.workdir_location: PathLike = workdir_location

    def add_workload(
        self,
        workload_location: str,
        model_path: str,
        workload_mode: WorkloadModes,
        workload_name: str,
        model_summary_file: Optional[str],
    ) -> str:
        """Add workload to Neural Insights."""
        if workload_mode == WorkloadModes.QUANTIZATION:
            workload = QuantizationWorkload()
        else:
            workload = Workload()
        workload.workload_name = workload_name
        workload.workload_location = workload_location
        workload.mode = workload_mode
        workload.model_path = model_path
        workload.model_summary_file = model_summary_file

        workload_manager = WorkloadManager(workdir_location=self.workdir_location)
        workload_manager.add_workload(workload)

        return workload.uuid

    def update_workload_status(self, workload_uuid: str, status: str) -> None:
        """Update status of specified workload."""
        workload_manager = WorkloadManager(workdir_location=self.workdir_location)
        workload_manager.update_workload_status(workload_uuid, status)

    def update_workload_accuracy_data(
        self,
        workload_uuid: str,
        baseline_accuracy: float,
        optimized_accuracy: float,
    ) -> None:
        """Update accuracy data of specified workload."""
        workload_manager = WorkloadManager(workdir_location=self.workdir_location)
        workload_manager.update_workload_accuracy_data(
            workload_uuid,
            baseline_accuracy,
            optimized_accuracy,
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Main endpoint for GUI."""
import os
import time
from functools import wraps
from threading import Thread
from typing import Any, Callable

from flask import Blueprint, Flask
from flask import Request as WebRequest
from flask import jsonify, render_template, request
from flask_cors import CORS
from flask_socketio import SocketIO
from werkzeug.serving import make_ssl_devcert
from werkzeug.wrappers import Response as WebResponse

from neural_insights.components.workload_manager.workload_manager import WorkloadManager
from neural_insights.utils.consts import CONFIG_REFRESH_PERIOD
from neural_insights.utils.exceptions import InternalException
from neural_insights.utils.logger import log
from neural_insights.web.communication import MessageQueue, Request
from neural_insights.web.configuration import Configuration
from neural_insights.web.router import Router
from neural_insights.web.service.response_generator import ResponseGenerator

templates_dir = os.path.abspath(
    os.path.join(
        os.path.dirname(__file__),
        "app",
    ),
)

app = Flask(
    __name__,
    static_folder=templates_dir,
    static_url_path="/",
    template_folder=templates_dir,
)
app_blueprint = Blueprint("Neural Insights", __name__)
socketio = SocketIO()

router = Router()

METHODS = ["GET", "POST"]

# Suppress TensorFlow messages
os.environ["TF_CPP_MIN_LOG_LEVEL"] = "3"

url_prefix: str = ""
workload_manager = None


def run_server(configuration: Configuration) -> None:
    """Run webserver on specified scheme, address and port."""
    addr = configuration.server_address
    server_port = configuration.server_port
    gui_port = configuration.gui_port
    token = configuration.token

    global url_prefix
    url_prefix = configuration.url_prefix

    cors_allowed_origins = f"{configuration.scheme}://{addr}:{gui_port}"

    app.config["JSON_SORT_KEYS"] = False
    app.register_blueprint(app_blueprint, url_prefix=url_prefix)
    app.secret_key = token
    CORS(app, origins=cors_allowed_origins)
    socketio.init_app(
        app,
        cors_allowed_origins=cors_allowed_origins,
        max_http_buffer_size=2000,
    )
    tls_args = get_tls_args(configuration)

    global workload_manager
    workload_manager = WorkloadManager(workdir_location=configuration.workdir)

    socketio.run(app, host=addr, port=server_port, **tls_args)


def get_tls_args(configuration: Configuration) -> dict:
    """Get TLS configuration."""
    if configuration.allow_insecure_connections:
        return {}

    if configuration.tls_certificate and configuration.tls_key:
        certfile = configuration.tls_certificate
        keyfile = configuration.tls_key
    else:
        os.makedirs(configuration.global_config_directory, mode=511, exist_ok=True)
        base_path = os.path.join(configuration.global_config_directory, "certificate")
        certfile = f"{base_path}.crt"
        keyfile = f"{base_path}.key"
        if not os.path.isfile(certfile) or not os.path.isfile(keyfile):
            certfile, keyfile = make_ssl_devcert(base_path)

    return {
        "certfile": certfile,
        "keyfile": keyfile,
        "do_handshake_on_connect": False,
    }


@app_blueprint.after_request
def block_iframe(response: WebResponse) -> WebResponse:
    """Block iframe and set others CSP."""
    response.headers["X-Frame-Options"] = "DENY"
    response.headers["Content-Security-Policy"] = (
        "frame-ancestors 'none'; font-src 'self'; img-src 'self'; script-src 'self'"
    )
    response.headers["Access-Control-Max-Age"] = "-1"
    return response


@app_blueprint.after_request
def block_sniffing(response: WebResponse) -> WebResponse:
    """Block MIME sniffing."""
    response.headers["X-Content-Type-Options"] = "nosniff"
    return response


def require_api_token(func: Callable) -> Any:
    """Validate authorization token."""

    @wraps(func)
    def check_token(*args: str, **kwargs: str) -> Any:
        """Validate that correct token was provided."""
        provided_token = request.headers.get(
            "Authorization",
            request.args.to_dict().get("token", None),
        )

        if not app.secret_key == provided_token:
            return (
                "Invalid token, please use the URL displayed by the server on startup",
                403,
            )

        return func(*args, **kwargs)

    return check_token


@app_blueprint.route("/", methods=METHODS)
def root() -> Any:
    """Serve JS application index."""
    return render_template("index.html", url_prefix=url_prefix)


@app_blueprint.route("/api/<path:subpath>", methods=METHODS)
@require_api_token
def handle_api_call(subpath: str) -> Any:
    """Handle API access."""
    try:
        parameters = build_parameters(subpath, request)
        response = router.handle(parameters)
        if isinstance(response, WebResponse):
            return response
        return jsonify(response.data)
    except Exception as err:
        if isinstance(err, InternalException):
            log.critical(err)
        return ResponseGenerator.from_exception(err)


@app_blueprint.route("/api/<path:subpath>", methods=["OPTIONS"])
def allow_api_call(subpath: str) -> Any:
    """Allow for API access."""
    return "OK"


@app.errorhandler(404)
def page_not_found(e: Any) -> Any:
    """Serve JS application index when no static file found."""
    return render_template(
        "index.html",
        url_prefix=url_prefix,
    )


@app_blueprint.after_request
def disable_cache(response: WebResponse) -> WebResponse:
    """Disable cache on all requests."""
    response.headers["Cache-Control"] = "no-cache, no-store, must-revalidate"
    response.headers["Pragma"] = "no-cache"
    response.headers["Expires"] = "0"
    response.headers["Cache-Control"] = "public, max-age=0"
    return response


def build_parameters(endpoint: str, request: WebRequest) -> Request:
    """Build domain object from flask request."""
    data = request.get_json() if request.is_json else request.args.to_dict(flat=False)
    return Request(request.method, endpoint, data)


def web_socket_publisher(web_socket: SocketIO) -> None:
    """Send messages from queue via web-socket to GUI."""
    queue = MessageQueue()
    while True:
        message = queue.get()
        web_socket.emit(
            message.subject,
            {"status": message.status, "data": message.data},
        )
        socketio.sleep(0)


publisher = Thread(
    target=web_socket_publisher,
    args=(socketio,),
)
publisher.daemon = True
publisher.start()


def config_watcher(period_in_s: int) -> None:
    """Observe config for changes and send notification when changed."""
    queue = MessageQueue()
    config_modification_time = None
    if workload_manager is not None:
        config_modification_time = os.stat(workload_manager.config_path).st_mtime
    while True:
        time.sleep(period_in_s)
        if workload_manager is None:
            continue
        config_path = workload_manager.config_path
        config_mod_time_new = os.stat(config_path).st_mtime
        if config_modification_time != config_mod_time_new:
            config_modification_time = config_mod_time_new
            queue.post_info("Config update", "Workload config has been modified.")


conf_watcher = Thread(
    target=config_watcher,
    args=(CONFIG_REFRESH_PERIOD,),
)
conf_watcher.daemon = True
conf_watcher.start()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tensorflow frozen pb model."""

from neural_insights.components.model.model_type_getter import get_model_type
from neural_insights.components.model.tensorflow.model import TensorflowModel as TFModel


class FrozenPbModel(TFModel):
    """Frozen pb model."""

    @staticmethod
    def supports_path(path: str) -> bool:
        """Check if given path is of supported model."""
        return "frozen_pb" == get_model_type(path)

    @property
    def supports_profiling(self) -> bool:
        """Check if profiling is supported for the model."""
        return True



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test Node."""

import unittest
import uuid

from neural_insights.components.graph.node import Node


def _get_random_string() -> str:
    """Create random string to be used."""
    return uuid.uuid4().hex


class TestNode(unittest.TestCase):
    """Test Node class."""

    def test_setting_parameters(self) -> None:
        """Test if parameters are correctly set."""
        id = _get_random_string()
        label = _get_random_string()

        node = Node(id=id, label=label)

        self.assertEqual(id, node.id)
        self.assertEqual(label, node.label)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test Graph Reader."""

import unittest
from unittest.mock import MagicMock, patch

from neural_insights.components.graph.graph import Graph
from neural_insights.components.graph.graph_reader import GraphReader


class TestGraphReader(unittest.TestCase):
    """Test GraphReader class."""

    @patch("neural_insights.components.graph.graph_reader.Collapser")
    @patch("neural_insights.components.graph.graph_reader.ModelRepository")
    def test_read(
        self,
        mocked_model_repository: MagicMock,
        mocked_collapser: MagicMock,
    ) -> None:
        """Test read."""
        model_path = "/path/to/model.file"
        expanded_groups = ["a", "b", "a/c"]

        model_graph = Graph()
        collapsed_graph = Graph()

        mocked_model = MagicMock("neural_insights.components.model.Model").return_value
        mocked_model.get_model_graph.return_value = model_graph

        mocked_model_repository.return_value.get_model.return_value = mocked_model

        mocked_collapser.return_value.collapse.return_value = collapsed_graph

        graph_reader = GraphReader()
        self.assertIs(collapsed_graph, graph_reader.read(model_path, expanded_groups))

        mocked_model_repository.assert_called_once()
        mocked_model_repository.return_value.get_model.assert_called_once_with(model_path)

        mocked_model.get_model_graph.assert_called_once()

        mocked_collapser.assert_called_once_with(expanded_groups)
        mocked_collapser.return_value.collapse.assert_called_once_with(model_graph)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Onnxrt Graph reader."""

import uuid
from typing import Any, Dict, List, Optional

from neural_insights.components.graph.attribute import Attribute
from neural_insights.components.graph.graph import Graph
from neural_insights.components.graph.node import Node
from neural_insights.components.model.model import Model
from neural_insights.utils.exceptions import NotFoundException


class OnnxrtReader:
    """Graph Reader for Onnxrt."""

    def __init__(self, model: Model) -> None:
        """Initialize object."""
        self._hidden_node_ids: Dict[str, bool] = {}
        self.model: Model = model

    def read(self) -> Graph:
        """Read a graph."""
        self._hidden_node_ids = {}

        from neural_insights.components.model.onnxrt.model import OnnxrtModel

        if not isinstance(self.model, OnnxrtModel):
            raise NotFoundException(f"{self.model.path} is not Onnxrt model.")

        onnx_nodes = self.model.nc_model_instance.nodes()
        graph = Graph()

        graph = self._add_boundary_nodes(graph)

        edges: dict = {}

        for node_def in onnx_nodes:
            node_name = node_def.name
            if node_name == "":
                node_name = f"{node_def.op_type}_{uuid.uuid4()}"

            if self._should_hide_node(node_def):
                self._hide_node(node_name)
                continue

            edges = self._add_edges_from_node(edges, node_def, node_name)

            graph.add_node(
                Node(
                    id=node_name,
                    label=node_def.op_type,
                    highlight=False,
                    properties={
                        "name": node_name,
                        "type": node_def.op_type,
                    },
                    attributes=self._convert_attributes(node_def),
                    groups=self._get_group_names(node_name),
                ),
            )

        self._process_edges(graph, edges)

        return graph

    def _get_group_names(self, node_id: str) -> List[str]:
        """Get a group names from Node starting from the top."""
        elements = node_id.split("/")
        names = []
        for idx in range(1, len(elements)):
            names.append("/".join(elements[0:idx]))
        return names

    def _should_hide_node(self, node_def: Any) -> bool:
        """Check if given node should be hidden."""
        if node_def.op_type not in ["Const", "Identity"]:
            return False

        if self._node_def_has_not_hidden_inputs(node_def):
            return False

        return True

    def _node_def_has_not_hidden_inputs(self, node_def: Any) -> bool:
        """Check if node has any visible input."""
        not_hidden_input_ids = list(
            filter(
                lambda input_node: input_node not in self._hidden_node_ids,
                node_def.input,
            ),
        )
        return len(not_hidden_input_ids) > 0

    def _hide_node(self, node_name: str) -> None:
        """Mark node as hidden."""
        self._hidden_node_ids[node_name] = True

    def _is_node_id_hidden(self, node_id: str) -> bool:
        """Check if provided node_id is hidden."""
        return node_id in self._hidden_node_ids

    def _convert_attributes(self, node_def: Any) -> List[Attribute]:
        """Convert NodeDef attributes to our format."""
        attributes = []

        for attribute in node_def.attribute:
            converted_attribute = self._convert_attribute(attribute)
            if converted_attribute is not None:
                attributes.append(converted_attribute)

        return attributes

    def _convert_attribute(self, attribute: Any) -> Optional[Attribute]:
        """Convert NodeDef attribute to our format."""
        from onnx.mapping import TENSOR_TYPE_TO_NP_TYPE

        attr_type = str(TENSOR_TYPE_TO_NP_TYPE.get(attribute.type, "UNKNOWN"))

        attr_float = attribute.f
        attr_floats = attribute.floats

        attr_int = attribute.i
        attr_ints = attribute.ints

        value = None
        if "int" in attr_type:
            value = attr_int
            if len(attr_ints) > 0:
                value = list(attr_ints)
        elif "float" in attr_type:
            value = attr_float
            if len(attr_floats) > 0:
                value = list(attr_floats)
        else:
            print("Attribute type not yet supported")

        return Attribute(
            attribute.name,
            str(TENSOR_TYPE_TO_NP_TYPE.get(attribute.type, "UNKNOWN")),
            value,
        )

    def _add_boundary_nodes(self, graph: Graph) -> Graph:
        """Add boundary nodes to graph."""
        from neural_insights.components.model.onnxrt.model import OnnxrtModel

        if not isinstance(self.model, OnnxrtModel):
            raise NotFoundException(f"{self.model.path} is not Onnxrt model.")

        onnx_graph = self.model.nc_model_instance.graph()
        boundary_nodes = {"Input node": onnx_graph.input, "Output node": onnx_graph.output}
        for node_type, nodes in boundary_nodes.items():
            for node in nodes:
                graph.add_node(
                    Node(
                        id=node.name,
                        label=node_type,
                        highlight=False,
                        properties={
                            "name": node.name,
                            "type": node_type,
                        },
                        attributes=[],
                        groups=self._get_group_names(node.name),
                    ),
                )
        return graph

    def _add_edges_from_node(self, edges: dict, node: Any, node_name: str) -> dict:
        """Add update edges from node data."""
        for edge in node.input:
            edge_data = edges.get(edge, {})
            input_nodes = edge_data.get("inputs", [])
            input_nodes.append(node_name)
            edge_data.update({"inputs": input_nodes})
            edges.update({edge: edge_data})

        for edge in node.output:
            edge_data = edges.get(edge, {})
            output_nodes = edge_data.get("outputs", [])
            output_nodes.append(node_name)
            edge_data.update({"outputs": output_nodes})
            edges.update({edge: edge_data})

        return edges

    def _process_edges(self, graph: Graph, edges: dict) -> None:
        """Process edges."""
        from neural_insights.components.model.onnxrt.model import OnnxrtModel

        if not isinstance(self.model, OnnxrtModel):
            raise NotFoundException(f"{self.model.path} is not Onnxrt model.")

        for edge, edge_data in edges.items():
            # Check if edge connects to input node
            input_nodes = [node.name for node in self.model.nc_model_instance.graph().input]
            if edge in input_nodes and "inputs" in edge_data:
                source_node = edge
                for target_node in edge_data["inputs"]:
                    graph.add_edge(
                        source_id=source_node,
                        target_id=target_node,
                    )

            # Check if edge connects to output node
            output_nodes = [node.name for node in self.model.nc_model_instance.graph().output]
            if edge in output_nodes and "outputs" in edge_data:
                target_node = edge
                for source_node in edge_data["outputs"]:
                    graph.add_edge(
                        source_id=source_node,
                        target_id=target_node,
                    )

            self._process_standard_edge(graph, edge_data)

    def _process_standard_edge(self, graph: Graph, edge_data: dict) -> None:
        """Process standard edges that contain both inputs and outputs."""
        if "inputs" not in edge_data or "outputs" not in edge_data:
            return
        for source_node in edge_data["outputs"]:
            for target_node in edge_data["inputs"]:
                if self._is_node_id_hidden(source_node) or self._is_node_id_hidden(
                    target_node,
                ):
                    continue
                graph.add_edge(
                    source_id=source_node,
                    target_id=target_node,
                )



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Graph reader."""
from typing import List, Optional, Tuple

from neural_insights.components.graph.collapser import Collapser
from neural_insights.components.graph.graph import Graph
from neural_insights.components.graph.node import Node
from neural_insights.components.model.repository import ModelRepository
from neural_insights.utils.exceptions import ClientErrorException


class GraphReader:
    """Model graph reader."""

    def read(self, model_path: str, expanded_groups: List[str]) -> Graph:
        """Return Graph for given model path."""
        model_repository = ModelRepository()
        model = model_repository.get_model(model_path)
        graph = model.get_model_graph()

        collapser = Collapser(expanded_groups)
        collapsed_graph = collapser.collapse(graph)

        return collapsed_graph

    def find_pattern_in_graph(
        self,
        model_path: str,
        op_name: str,
        pattern: List[str],
    ) -> Tuple[Graph, List[str]]:
        """Search graph for specific nodes pattern."""
        model_repository = ModelRepository()
        model = model_repository.get_model(model_path)
        graph = model.get_model_graph()

        op_data: Optional[Node] = None
        for node in graph.nodes:
            if node.id == op_name:
                op_data = node
                break

        if op_data is None:
            raise ClientErrorException(f"Could not find {op_name} in graph.")

        expanded_groups = op_data.groups

        collapser = Collapser(expanded_groups)
        collapsed_graph = collapser.collapse(graph)

        collapsed_graph.highlight_pattern(op_name, pattern)

        return collapsed_graph, expanded_groups



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tensorflow Graph reader."""

from typing import Any, Dict, List, Optional

from neural_insights.components.graph.attribute import Attribute
from neural_insights.components.graph.graph import Graph
from neural_insights.components.graph.node import Node
from neural_insights.components.model.model import Model
from neural_insights.utils.exceptions import NotFoundException


class TensorflowReader:
    """Graph Reader for Tensorflow."""

    def __init__(self, model: Model) -> None:
        """Initialize object."""
        self._hidden_node_ids: Dict[str, bool] = {}
        self.model: Model = model

    def read(self) -> Graph:
        """Read a graph."""
        self._hidden_node_ids = {}

        from neural_insights.components.model.tensorflow.model import TensorflowModel

        if not isinstance(self.model, TensorflowModel):
            raise NotFoundException(f"{self.model.path} is not Tensorflow model.")

        graph_def = self.model.nc_model_instance.graph_def

        graph = Graph()

        for node_def in graph_def.node:
            if self._should_hide_node(node_def):
                self._hide_node(node_def)
                continue

            current_node_id = node_def.name

            graph.add_node(
                Node(
                    id=current_node_id,
                    label=node_def.op,
                    highlight=False,
                    properties={
                        "name": node_def.name,
                        "type": node_def.op,
                    },
                    attributes=self._convert_attributes(node_def),
                    groups=self._get_group_names(current_node_id),
                ),
            )

            for input_node_name in node_def.input:
                if self._is_node_id_hidden(input_node_name) or self._is_node_id_hidden(
                    current_node_id,
                ):
                    continue
                graph.add_edge(
                    source_id=input_node_name,
                    target_id=current_node_id,
                )

        return graph

    def _get_group_names(self, node_id: str) -> List[str]:
        """Get a group names from Node starting from the top."""
        elements = node_id.split("/")
        names = []
        for idx in range(1, len(elements)):
            names.append("/".join(elements[0:idx]))
        return names

    def _should_hide_node(self, node_def: Any) -> bool:
        """Check if given node should be hidden."""
        if node_def.op not in ["Const", "Identity"]:
            return False

        if self._node_def_has_not_hidden_inputs(node_def):
            return False

        return True

    def _node_def_has_not_hidden_inputs(self, node_def: Any) -> bool:
        """Check if node has any visible input."""
        not_hidden_input_ids = list(
            filter(
                lambda input_node: input_node not in self._hidden_node_ids,
                node_def.input,
            ),
        )
        return len(not_hidden_input_ids) > 0

    def _hide_node(self, node: Any) -> None:
        """Mark node as hidden."""
        self._hidden_node_ids[node.name] = True

    def _is_node_id_hidden(self, node_id: str) -> bool:
        """Check if provided node_id is hidden."""
        return node_id in self._hidden_node_ids

    def _convert_attributes(self, node_def: Any) -> List[Attribute]:
        """Convert NodeDef attributes to our format."""
        attributes = []

        for attribute_name in node_def.attr:
            converted_attribute = self._convert_attribute(
                attribute_name,
                node_def.attr[attribute_name],
            )
            if converted_attribute is not None:
                attributes.append(converted_attribute)

        return attributes

    def _convert_attribute(self, name: str, value: Any) -> Optional[Attribute]:
        """Convert NodeDef attribute to our format."""
        from tensorflow.python.framework.dtypes import _TYPE_TO_STRING

        if name == "shape":
            shape = [dim.size for dim in value.shape.dim]
            return Attribute(
                name,
                "shape",
                shape,
            )

        if 0 != len(value.s):
            return Attribute(name, "string", str(value.s.decode("utf-8")))
        if 0 != value.type:
            return Attribute(
                name,
                "type",
                _TYPE_TO_STRING.get(value.type, "UNKNOWN"),
            )
        return None



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Objects to communicate between domain logic and outside layers."""

from queue import Queue
from typing import Any, Dict, List, Union


class Request:
    """Domain defined parameters."""

    def __init__(self, method: str, operation: str, data: dict) -> None:
        """Initialize properties."""
        self.method: str = method
        self.operation: str = operation
        self.data: dict = data


class Response:
    """Domain defined response."""

    def __init__(self) -> None:
        """Initialize properties."""
        self.data: Union[Dict[str, Any], List[Dict[str, Any]]] = {}
        self.command: Dict[str, Any] = {}


def create_simple_response(
    data: Union[Dict[str, Any], List[Dict[str, Any]]],
) -> Response:
    """Create new Response object with only data set."""
    response = Response()
    response.data = data
    return response


class Message:
    """Message used to send data to GUI."""

    def __init__(self, status: str, subject: str, data: Any):
        """Initialize message."""
        self._status: str = status
        self._subject: str = subject
        self._data: Any = data

    @property
    def subject(self) -> str:
        """Get the subject."""
        return self._subject

    @property
    def status(self) -> str:
        """Get the status."""
        return self._status

    @property
    def data(self) -> Any:
        """Get the data."""
        return self._data


class MessageQueue:
    """Queue for passing messages to GUI."""

    _queue: Queue = Queue()

    def post_failure(self, subject: str, data: Any) -> None:
        """Post failure message."""
        self._queue.put(Message("failure", subject, data))

    def post_success(self, subject: str, data: Any) -> None:
        """Post success message."""
        self._queue.put(Message("success", subject, data))

    def post_error(self, subject: str, data: Any) -> None:
        """Post error message."""
        self._queue.put(Message("error", subject, data))

    def post_info(self, subject: str, data: Any) -> None:
        """Post info message."""
        self._queue.put(Message("info", subject, data))

    def get(self) -> Message:
        """Wait for message and return it."""
        return self._queue.get()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Diagnosis class factory."""
from neural_insights.components.diagnosis.diagnosis import Diagnosis
from neural_insights.components.diagnosis.onnx_diagnosis.onnxrt_diagnosis import OnnxRtDiagnosis
from neural_insights.components.diagnosis.pytorch_diagnosis.pytorch_diagnosis import PyTorchDiagnosis
from neural_insights.components.diagnosis.tensorflow_diagnosis.tensorflow_diagnosis import TensorflowDiagnosis
from neural_insights.components.workload_manager.workload import Workload
from neural_insights.utils.consts import Frameworks
from neural_insights.utils.exceptions import InternalException
from neural_insights.utils.logger import log


class DiagnosisFactory:
    """Optimization factory."""

    @staticmethod
    def get_diagnosis(
        workload: Workload,
    ) -> Diagnosis:
        """Get diagnosis for specified framework."""
        diagnosis_map = {
            Frameworks.ONNX: OnnxRtDiagnosis,
            Frameworks.TF: TensorflowDiagnosis,
            Frameworks.PT: PyTorchDiagnosis,
        }
        diagnosis = diagnosis_map.get(workload.framework, None)
        if diagnosis is None:
            raise InternalException(
                f"Could not find diagnosis class for {workload.framework.value} framework.",
            )
        log.debug(f"Initializing {diagnosis.__name__} class.")
        return diagnosis(workload)



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Singleton metaclass."""

from typing import Any


class Singleton(type):
    """Singleton class."""

    _instances: dict = {}

    def __call__(cls, *args: list, **kwargs: dict) -> Any:
        """Get or create singleton instance."""
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Weights details class."""
from typing import Any, Dict, Optional

from neural_compressor.utils.utility import mse_metric_gap
from neural_insights.utils.json_serializer import JsonSerializer
from neural_insights.utils.utils import check_module


class WeightsDetails(JsonSerializer):
    """Weights details class."""

    def __init__(self, op_name: str, input_tensor_data, optimized_tensor_data):
        """Initialize Weights details."""
        super().__init__()
        self.op_name: str = op_name
        self.mse: Optional[float] = mse_metric_gap(
            input_tensor_data,
            optimized_tensor_data,
        )
        self.input_stats: WeightsStatistics = WeightsStatistics(input_tensor_data)
        self.optimized_stats: WeightsStatistics = WeightsStatistics(optimized_tensor_data)

    def serialize(
        self,
        serialization_type: str = "default",
    ) -> Dict[str, Any]:
        """Serialize Weights details."""
        return {
            "OP name": self.op_name,
            "MSE": self.mse,
            "Input model": self.input_stats.serialize(),
            "Optimized model": self.optimized_stats.serialize(),
        }


class WeightsStatistics(JsonSerializer):
    """Weights statistics class."""

    def __init__(self, tensor_data) -> None:
        """Initialize Weights details."""
        check_module("numpy")
        import numpy as np

        super().__init__()
        self.min: float = np.min(tensor_data)
        self.max: float = np.max(tensor_data)
        self.mean: float = np.mean(tensor_data)
        self.std: float = np.std(tensor_data)
        self.var: float = np.var(tensor_data)

    def serialize(
        self,
        serialization_type: str = "default",
    ) -> Dict[str, Any]:
        """Serialize Weights details."""
        return {
            "Min weight": self.min,
            "Max weight": self.max,
            "Mean": self.mean,
            "Standard deviation": self.std,
            "Variance": self.var,
        }



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Graph Node Attribute."""

from typing import Any

from neural_insights.utils.json_serializer import JsonSerializer


class Attribute(JsonSerializer):
    """Graph Node Attribute."""

    def __init__(self, name: str, attribute_type: str, value: Any):
        """Construct the object."""
        super().__init__()
        self.name = name
        self.attribute_type = attribute_type
        self.value = value



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Model type getter."""

from neural_compressor.model.model import get_model_type as nc_get_model_type
from neural_insights.utils.expiring_dict import ExpiringDict

model_type_cache = ExpiringDict(ttl=600)


def get_model_type(model_path: str) -> str:
    """Get model_type using local cache."""
    try:
        return model_type_cache[model_path]
    except KeyError:
        try:
            model_type = nc_get_model_type(model_path)
        except Exception:
            model_type = "not a model"
        model_type_cache[model_path] = model_type
        return model_type



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Constant values."""

import os
from enum import Enum

WORKDIR_LOCATION = os.path.join(os.environ.get("HOME", ""), ".neural_compressor")

CONFIG_REFRESH_PERIOD = 5


class WorkloadModes(Enum):
    """Workload mode enumeration."""

    BENCHMARK = "benchmark"
    QUANTIZATION = "quantization"


class Domains(Enum):
    """Model domains enumeration."""

    NONE = ""
    IMAGE_RECOGNITION = "Image Recognition"
    OBJECT_DETECTION = "Object Detection"
    NLP = "Neural Language Processing"
    RECOMMENDATION = "Recommendation"


class DomainFlavours(Enum):
    """Domain flavours enumeration."""

    NONE = ""
    SSD = "SSD"
    YOLO = "Yolo"


class Frameworks(Enum):
    """Supported frameworks enumeration."""

    TF = "TensorFlow"
    ONNX = "ONNXRT"
    PT = "PyTorch"


class WorkloadStatus(Enum):
    """Workload status enumeration."""

    NEW = "new"
    WIP = "wip"
    SUCCESS = "success"
    FAILURE = "failure"



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The OnnxRT diagnosis class."""
from typing import Optional

from neural_insights.components.diagnosis.diagnosis import Diagnosis
from neural_insights.components.model.onnxrt.model import OnnxrtModel
from neural_insights.components.model.repository import ModelRepository
from neural_insights.components.workload_manager.workload import Workload


class OnnxRtDiagnosis(Diagnosis):
    """OnnxRuntime diagnosis class."""

    def __init__(self, workload: Workload):
        """Initialize OnnxRtDiagnosis."""
        super().__init__(workload)
        self._model: Optional[OnnxrtModel] = None

    @property
    def model(self) -> OnnxrtModel:
        """Get Neural Compressor Model instance."""
        self._ensure_model()
        return self._model  # type: ignore

    def _ensure_model(self) -> None:
        """Create Neural Insights Model instance if needed."""
        if self._model is not None:
            return
        model_repository = ModelRepository()
        self._model = model_repository.get_model(self.model_path)  # type: ignore



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TensorFlow model utils."""

from typing import Any, List


def get_input_shape(graph_def: Any, fix_dynamic_shape: int) -> List[int]:
    """Get input shape of passed graph."""
    try:
        import tensorflow.compat.v1 as tf_v1
    except ImportError:
        import tensorflow as tf_v1

    graph = tf_v1.Graph()
    with graph.as_default():  # pylint: disable=not-context-manager
        tf_v1.import_graph_def(graph_def, name="")

    node_dict = {}
    for node in graph.as_graph_def().node:  # pylint: disable=no-member
        if node.op == "Placeholder":
            node_dict = {"type": tf_v1.DType(node.attr["dtype"].type).name}

            if node_dict["type"] != "bool":
                # convert shape to list
                node["shape"] = _convert_shape_to_list(node, fix_dynamic_shape, tf_v1)
            else:  # deal with bool dtype inputs, now assign bool dtype input False value
                node_dict["shape"] = None
                node_dict["value"] = False

    return node_dict["shape"]


def _convert_shape_to_list(
    node: Any,
    fix_dynamic_shape: int,
    tf_module: Any,
) -> list:
    """Convert tensorflow shape to list."""
    try:
        _shape = list(tf_module.TensorShape(node.attr["shape"].shape))
        if tf_module.__version__ >= "2.0.0":
            shape = [item if item is not None else fix_dynamic_shape for item in _shape]
        else:
            shape = [item.value if item.value is not None else fix_dynamic_shape for item in _shape]
        # if shape dimension > 1, suppose first dimension is batch-size
        if isinstance(shape, list) and len(shape) > 1:
            return shape[1:]
        return shape
    except ValueError:
        _shape = [fix_dynamic_shape, fix_dynamic_shape, 3]
        return _shape



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Node class."""

from typing import Any, Dict, List

from neural_insights.components.graph.attribute import Attribute
from neural_insights.utils.json_serializer import JsonSerializer


class Node(JsonSerializer):
    """Single Node in graph."""

    TYPE_SIMPLE_NODE = "node"

    def __init__(
        self,
        id: str,
        label: str,
        highlight: bool = False,
        properties: Dict[str, Any] = {},
        attributes: List[Attribute] = [],
        groups: List[str] = [],
    ) -> None:
        """Construct object."""
        super().__init__()
        self.id = id
        self.label = label
        self.highlight = highlight
        self.properties = properties
        self.attributes = attributes
        self.groups = groups
        self.node_type = self.TYPE_SIMPLE_NODE
        self._skip.append("groups")


class GroupNode(Node):
    """Node being a group of Nodes in Graph."""

    TYPE_GROUP_NODE = "group_node"

    def __init__(
        self,
        id: str,
        group_name: str,
    ) -> None:
        """Construct object."""
        super().__init__(
            id=id,
            label=group_name,
        )
        self.group = group_name
        self.node_type = self.TYPE_GROUP_NODE



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The abstract model class."""

from abc import ABC, abstractmethod
from typing import Any, List, Optional

from neural_insights.components.graph.graph import Graph
from neural_insights.components.model.domain import Domain
from neural_insights.components.model.shape import Shape
from neural_insights.utils.consts import DomainFlavours, Domains
from neural_insights.utils.logger import log
from neural_insights.utils.utils import get_size


class Model(ABC):
    """Abstract model class."""

    def __init__(self, path: str) -> None:
        """Initialize object."""
        self.ensure_supported_path(path)
        self.path = path
        self.size = get_size(self.path)
        self.framework = None

    def get_input_nodes(self) -> Optional[List[Any]]:
        """Get model input nodes."""
        self.guard_requirements_installed()
        return None

    def get_output_nodes(self) -> Optional[List[Any]]:
        """Get model output nodes."""
        self.guard_requirements_installed()
        return None

    def ensure_supported_path(self, path: str) -> None:
        """Make sure that provided path is to supported model."""
        if not self.supports_path(path):
            klass = self.__class__
            model_name = ".".join([klass.__module__, klass.__qualname__])
            raise AttributeError(f"Model path: {path} is not supported by {model_name} class.")

    def get_model_graph(self) -> Graph:
        """Get model graph."""
        raise NotImplementedError(f"Reading graph for model {self.path} is not supported.")

    def get_tensors_info(self) -> dict:
        """Get information about tensors."""
        raise NotImplementedError(
            f"Getting tensors informarmation for model {self.path} is not supported.",
        )

    @property
    def domain(self) -> Domain:
        """Get model domain."""
        try:
            node_names = {node.id for node in self.get_model_graph().nodes}
        except Exception:
            return Domain()

        def has_name_part(name_part: str) -> bool:
            """Check if a node exists with given name_part."""
            return bool([name for name in node_names if name_part in name])

        def has_all_name_parts(name_parts: List[str]) -> bool:
            """Check if there is a node for every name_part."""
            missing = [name_part for name_part in name_parts if not has_name_part(name_part)]
            return not missing

        if has_all_name_parts(["bboxes", "scores", "classes", "ssd"]):
            return Domain(
                domain=Domains.OBJECT_DETECTION.value,
                domain_flavour=DomainFlavours.SSD.value,
            )
        if has_all_name_parts(["boxes", "yolo"]):
            return Domain(
                domain=Domains.OBJECT_DETECTION.value,
                domain_flavour=DomainFlavours.YOLO.value,
            )
        if has_all_name_parts(["boxes", "scores", "classes"]):
            return Domain(domain=Domains.OBJECT_DETECTION.value)
        if has_all_name_parts(["resnet"]):
            return Domain(domain=Domains.IMAGE_RECOGNITION.value)
        return Domain()

    @property
    def input_shape(self) -> Shape:
        """Try to detect data shape."""
        return Shape(trusted=True)

    @property
    def shape_elements_order(self) -> List[str]:
        """Get order of input shape channels."""
        raise NotImplementedError

    @staticmethod
    @abstractmethod
    def get_framework_name() -> str:
        """Get the name of framework."""

    @staticmethod
    @abstractmethod
    def supports_path(path: str) -> bool:
        """Check if given path is of supported model."""

    @property
    def supports_profiling(self) -> bool:
        """Check if profiling is supported for the model."""
        return False

    @property
    def supports_graph(self) -> bool:
        """Check if it is possible to display model's graph."""
        try:
            self.get_model_graph()
            return True
        except Exception:
            log.debug("Could not load graph of model.")
        return False

    @property
    def supports_pruning(self) -> bool:
        """Check if pruning is supported for the model."""
        return False

    @abstractmethod
    def guard_requirements_installed(self) -> None:
        """Ensure all requirements are installed."""



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""JsonSerializer module."""

import re
from enum import Enum
from typing import Any, Dict, List, Optional, Union

from neural_insights.utils.logger import log


class JsonSerializer:
    """Dict serializable class."""

    def __init__(self) -> None:
        """Initialize json serializable class."""
        # List of variable names that will
        # be skipped during serialization
        self._skip = ["_skip"]

    def __eq__(self, other: Any) -> bool:
        """Compare self to other instance."""
        if type(self) is not type(other):
            # don't attempt to compare against unrelated types
            raise NotImplementedError

        return self.serialize() == other.serialize()

    def serialize(
        self,
        serialization_type: str = "default",
    ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
        """Serialize class to dict.

        :param serialization_type: serialization type, defaults to "default"
        :type serialization_type: str, optional
        :return: serialized class
        :rtype: Union[dict, List[dict]]
        """
        result = {}
        for key, value in self.__dict__.items():
            if key in self._skip:
                continue
            if value is None:
                continue

            variable_name = re.sub(r"^_", "", key)
            getter_value = value
            try:
                getter_value = getattr(self, variable_name)
            except AttributeError:
                log.warning(f"Found f{key} attribute without {variable_name} getter.")

            serialized_value = self._serialize_value(
                getter_value,
                serialization_type,
            )

            if serialized_value is not None:
                result[variable_name] = serialized_value

        return result

    def _serialize_value(self, value: Any, serialization_type: str) -> Any:
        """Serialize single value."""
        if isinstance(value, list):
            return self._serialize_list(value, serialization_type)
        if isinstance(value, dict):
            return self._serialize_dict(value, serialization_type)
        else:
            return self.serialize_item(value, serialization_type)

    def _serialize_list(
        self,
        value: list,
        serialization_type: str,
    ) -> Optional[List[Any]]:
        """Serialize list."""
        serialized_list = []

        for item in value:
            serialized_item = self.serialize_item(item, serialization_type)
            if serialized_item is not None:
                serialized_list.append(serialized_item)

        if len(serialized_list) == 0:
            return None

        return serialized_list

    def _serialize_dict(
        self,
        value: dict,
        serialization_type: str,
    ) -> Optional[Dict[str, Any]]:
        """Serialize dict."""
        serialized_dict = {}

        for key in value.keys():
            serialized_item = self.serialize_item(value[key], serialization_type)
            if serialized_item is not None:
                serialized_dict[key] = serialized_item

        if len(serialized_dict.keys()) == 0:
            return None

        return serialized_dict

    @staticmethod
    def serialize_item(value: Any, serialization_type: str = "default") -> Any:
        """Serialize objects that don't support json dump.

        i.e datetime object can't be serialized to JSON format and throw an TypeError exception
        TypeError: datetime.datetime(2016, 4, 8, 11, 22, 3, 84913) is not JSON serializable
        To handle that override method serialize_item to convert object
            >>> serialize_item(datetime)
            "2016-04-08T11:22:03.084913"

        For all other cases it should return serializable object i.e. str, int float

        :param value: Any type
        :param serialization_type: serialization type
        :return: Value that can be handled by json.dump
        """
        if issubclass(type(value), JsonSerializer):
            # pylint: disable=maybe-no-member
            serialized_value = value.serialize(serialization_type)
            return serialized_value
        if isinstance(value, Enum):
            return value.value
        return value



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Edge class."""

from neural_insights.components.graph.node import Node
from neural_insights.utils.json_serializer import JsonSerializer


class Edge(JsonSerializer):
    """Edge in graph."""

    def __init__(self, source: Node, target: Node) -> None:
        """Construct object."""
        super().__init__()
        self._source = source
        self._target = target

    @property
    def source(self) -> str:
        """Return source Node id."""
        return self._source.id

    @property
    def target(self) -> str:
        """Return target Node id."""
        return self._target.id



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Dict with items expiring after given time."""
import time
from collections import UserDict
from typing import Any, Optional


class ExpiringDictItem:
    """Item that knows it it's already expired."""

    def __init__(self, value: Any, expires_at: float):
        """Create object."""
        self.value = value
        self.expires_at = expires_at

    def is_expired(self) -> bool:
        """Check if item is already expired."""
        return time.time() > self.expires_at


class ExpiringDict(UserDict):
    """Dict with items expiring after given time."""

    def __init__(self, initial_value: Optional[dict] = None, ttl: int = 120) -> None:
        """Create object."""
        super().__init__()
        self.ttl = ttl
        if initial_value is None:
            initial_value = {}

        for key, value in initial_value.items():
            self[key] = value

    def __setitem__(self, key: str, item: Any) -> None:
        """Add item to dict."""
        super().__setitem__(key, self._create_item(value=item))

    def __getitem__(self, key: str) -> Any:
        """Get item from dict."""
        item: ExpiringDictItem = super().__getitem__(key)
        if item.is_expired():
            raise KeyError(key)
        return item.value

    def _create_item(self, value: Any) -> ExpiringDictItem:
        """Create item for collection."""
        return ExpiringDictItem(value=value, expires_at=time.time() + self.ttl)



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The workload_manager module contains code used for Neural Insights workloads management."""

import json
import os
import shutil
from os import PathLike
from typing import List, Optional

from neural_insights.components.workload_manager.quantization_workload import AccuracyData, QuantizationWorkload
from neural_insights.components.workload_manager.workload import Workload
from neural_insights.utils.consts import WORKDIR_LOCATION, WorkloadModes, WorkloadStatus
from neural_insights.utils.exceptions import ClientErrorException, InternalException
from neural_insights.utils.json_serializer import JsonSerializer
from neural_insights.utils.logger import log
from neural_insights.utils.singleton import Singleton


class WorkloadManager(JsonSerializer, metaclass=Singleton):
    """Workload Manager class."""

    def __init__(self, workdir_location: Optional[PathLike] = None):
        """Initialize WorkloadManager."""
        super().__init__()
        if workdir_location is None:
            log.debug("Using default workdir location.")
            workdir_location = WORKDIR_LOCATION
        self.config_path = os.path.join(workdir_location, "neural_insights", "config.json")
        self._version: int = 1
        self._workloads: List[Workload] = []

        self.load_workloads()
        self.dump_config()

    def add_workload(self, workload: Workload) -> None:
        """Add workload."""
        self._workloads.append(workload)
        self.dump_config()

    def update_workload_status(self, workload_uuid: str, status: str) -> None:
        """Update workload status."""
        try:
            new_status = WorkloadStatus(status)
        except ValueError:
            raise ClientErrorException("Could not parse status.")
        workload = self.get_workload(workload_uuid)
        current_status = workload.status
        self.validate_status_flow(current_status, new_status)
        workload.status = new_status
        self.dump_config()

    def update_workload_accuracy_data(
        self,
        workload_uuid: str,
        baseline_accuracy: float,
        optimized_accuracy: float,
    ) -> None:
        """Update workload status."""
        workload = self.get_workload(workload_uuid)
        if not isinstance(workload, QuantizationWorkload):
            raise ClientErrorException(
                "Accuracy data is only available in quantization workloads.",
            )

        accuracy_data = AccuracyData(
            {
                "baseline_accuracy": baseline_accuracy,
                "optimized_accuracy": optimized_accuracy,
            },
        )
        workload.accuracy_data = accuracy_data
        self.dump_config()

    @staticmethod
    def validate_status_flow(
        prev_status: WorkloadStatus,
        new_status: WorkloadStatus,
    ) -> None:
        """Validate status flow."""
        status_indices = {
            WorkloadStatus.NEW: 0,
            WorkloadStatus.WIP: 1,
            WorkloadStatus.SUCCESS: 2,
            WorkloadStatus.FAILURE: 3,
        }
        status_connections = [(0, 1), (1, 2), (1, 3)]

        # Initialize adjacency matrix with zeros
        adjacency_matrix = [[0 for _ in status_indices] for _ in status_indices]
        for source, target in status_connections:
            adjacency_matrix[source][target] = 1

        prev_status_indice = status_indices[prev_status]
        new_status_indice = status_indices[new_status]
        if adjacency_matrix[prev_status_indice][new_status_indice] != 1:
            raise ClientErrorException(
                f"Changing status from {prev_status.value} to {new_status.value} is not allowed.",
            )

    def load_workloads(self) -> None:
        """Read workloads from config file."""
        if not os.path.exists(self.config_path):
            return
        with open(self.config_path, "r") as config_file:
            config_data = json.load(config_file)
        config_version = int(config_data.get("version"))
        if config_version != self._version:
            raise InternalException(
                f"Incompatible config version has been found. "
                f"Expected version: {self._version}, found {config_version}.",
            )
        workloads_config = config_data.get("workloads", [])
        if not isinstance(workloads_config, list):
            raise InternalException(
                "Incompatible format of workloads. Workloads should be a list.",
            )
        self._workloads = []
        for workload_config in workloads_config:
            if workload_config.get("mode") == WorkloadModes.QUANTIZATION.value:
                workload = QuantizationWorkload(workload_config)
            else:
                workload = Workload(workload_config)
            self._workloads.append(workload)

    def dump_config(self) -> None:
        """Dump workloads to config file."""
        serialized_workloads = [workload.serialize() for workload in self._workloads]
        data = {
            "version": self._version,
            "workloads": serialized_workloads,
        }

        os.makedirs(os.path.dirname(self.config_path), exist_ok=True)
        with open(self.config_path, "w") as config_file:
            json.dump(data, config_file)

    @property
    def workloads(self) -> List[Workload]:
        """Get workloads list."""
        self.load_workloads()
        return self._workloads

    def get_workload(self, workload_uuid: str) -> Workload:
        """Get workload from workloads list."""
        for workload in self.workloads:
            if workload.uuid == workload_uuid:
                return workload
        raise ClientErrorException("Could not find workload with specified ID.")

    def remove_workload(self, workload_uuid: str) -> str:
        """Remove workload from workloads list."""
        for index, workload in enumerate(self.workloads):
            if workload.uuid == workload_uuid:
                shutil.rmtree(workload.workload_location, ignore_errors=True)
                del self._workloads[index]
                self.dump_config()
                return workload.uuid
        raise ClientErrorException("Could not find workload with specified ID.")



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test Onnx Model."""

import unittest
from unittest.mock import MagicMock, call, patch

from neural_insights.components.graph.graph import Graph
from neural_insights.components.model.onnxrt.model import (
    OnnxrtModel,
    remove_number_of_samples_from_shape,
)
from neural_insights.utils.consts import Frameworks


class TestOnnxrtModel(unittest.TestCase):
    """Test OnnxrtModel class."""

    def test_get_framework_name(self) -> None:
        """Test getting correct framework name."""
        self.assertEqual(Frameworks.ONNX.value, OnnxrtModel.get_framework_name())

    def test_supports_correct_path(self) -> None:
        """Test getting correct framework name."""
        self.assertTrue(OnnxrtModel.supports_path("/path/to/model.onnx"))

    def test_supports_incorrect_path(self) -> None:
        """Test getting correct framework name."""
        self.assertFalse(OnnxrtModel.supports_path("/path/to/model.pb"))

    @patch("neural_insights.components.model.onnxrt.model.check_module")
    def test_guard_requirements_installed(self, mocked_check_module: MagicMock) -> None:
        """Test guard_requirements_installed."""
        model = OnnxrtModel("/path/to/model.onnx")

        model.guard_requirements_installed()

        mocked_check_module.assert_has_calls([call("onnx"), call("onnxruntime")])

    @patch("neural_insights.components.model.onnxrt.model.check_module")
    def test_get_input_nodes(self, mocked_check_module: MagicMock) -> None:
        """Test getting input nodes."""
        model = OnnxrtModel("/path/to/model.onnx")
        self.assertIsNone(model.get_input_nodes())

    @patch("neural_insights.components.model.onnxrt.model.check_module")
    def test_get_output_nodes(self, mocked_check_module: MagicMock) -> None:
        """Test getting output nodes."""
        model = OnnxrtModel("/path/to/model.onnx")
        self.assertIsNone(model.get_output_nodes())

    @patch("neural_insights.components.model.onnxrt.model.check_module")
    def test_get_input_and_output_nodes(self, mocked_check_module: MagicMock) -> None:
        """Test getting input nodes."""
        model = OnnxrtModel("/path/to/model.onnx")
        self.assertIsNone(model.get_input_nodes())
        self.assertIsNone(model.get_output_nodes())

    def test_ensure_supported_path(self) -> None:
        """Test ensure_supported_path."""
        with self.assertRaisesRegex(
            AttributeError,
            "Model path: /path/to/model.pb is not supported by "
            "neural_insights.components.model.onnxrt.model.OnnxrtModel class.",
        ):
            OnnxrtModel("/path/to/model.pb")

    @patch(
        "neural_insights.components.model.onnxrt.model.OnnxrtReader",
        autospec=True,
    )
    def test_get_model_graph(self, mocked_onnxrt_graph_reader: MagicMock) -> None:
        """Test getting Graph of a model."""
        expected = Graph()

        mocked_onnxrt_graph_reader.return_value.read.return_value = expected

        model = OnnxrtModel("/path/to/model.onnx")

        self.assertEqual(expected, model.get_model_graph())

        mocked_onnxrt_graph_reader.assert_called_once_with(model)

    def test_shape_elements_order(self) -> None:
        """Test getting shape elements order."""
        model = OnnxrtModel("/path/to/model.onnx")
        self.assertListEqual(model.shape_elements_order, ["channels", "height", "width"])

    def test_remove_number_of_samples_from_shape(self) -> None:
        """Test removing number of samples from shape."""
        shape = "1,3,224,224"
        actual = remove_number_of_samples_from_shape(shape)

        expected = "3,224,224"
        self.assertEqual(expected, actual)

    def test_remove_number_of_samples_from_standard_shape(self) -> None:
        """Test removing number of samples from shape."""
        shape = "1,2,3"
        actual = remove_number_of_samples_from_shape(shape)

        expected = "1,2,3"
        self.assertEqual(expected, actual)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The quantization_workload module for Neural Insights quantization workloads."""
from typing import Any, Dict, Optional

from neural_insights.components.workload_manager.workload import Workload
from neural_insights.utils.json_serializer import JsonSerializer


class QuantizationWorkload(Workload):
    """QuantizationWorkload class."""

    def __init__(self, data: Optional[dict] = None):
        """Initialize Quantization Workload."""
        super().__init__(data)
        if data is None:
            data = {}
        self.accuracy_data = AccuracyData(data.get("accuracy_data", None))

    def serialize(self, serialization_type: str = "default") -> Dict[str, Any]:
        """Serialize Workload class."""
        serialized_workload = super().serialize(serialization_type)
        serialized_workload.update({"accuracy_data": self.accuracy_data.serialize()})
        return serialized_workload


class AccuracyData(JsonSerializer):
    """AccuracyData class."""

    def __init__(self, data: Optional[dict] = None):
        """Initialize Accuracy Data."""
        super().__init__()
        if data is None:
            data = {}
        self._baseline_accuracy: Optional[float] = data.get("baseline_accuracy", None)
        self._optimized_accuracy: Optional[float] = data.get("optimized_accuracy", None)

    @property
    def baseline_accuracy(self) -> Optional[float]:
        """Get baseline accuracy.

        Returns: baseline accuracy value
        """
        return self._baseline_accuracy

    @property
    def optimized_accuracy(self) -> Optional[float]:
        """Get optimized accuracy.

        Returns: optimized accuracy value
        """
        return self._optimized_accuracy

    @property
    def ratio(self) -> Optional[float]:
        """Get accuracy ratio.

        Returns: accuracy ratio if baseline and optimized accuracy are present
                 Otherwise returns None
        """
        if self.optimized_accuracy in [None, 0] or self.baseline_accuracy in [None, 0]:
            return None

        return (self.optimized_accuracy - self.baseline_accuracy) / self.baseline_accuracy

    def serialize(self, serialization_type: str = "default") -> Dict[str, Any]:
        """Serialize AccuracyData class."""
        return {
            "baseline_accuracy": self.baseline_accuracy,
            "optimized_accuracy": self.optimized_accuracy,
            "ratio": self.ratio,
        }



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Configuration module for Neural Insights server."""

import argparse
import logging
import os
import secrets
import socket
import sys
from typing import Dict

from numpy.random import randint

from neural_compressor.utils.utility import singleton
from neural_insights.utils.consts import WORKDIR_LOCATION
from neural_insights.utils.exceptions import NotFoundException
from neural_insights.utils.logger import log
from neural_insights.utils.utils import determine_ip


@singleton
class Configuration:
    """Configuration object for Neural Insights server."""

    PORT_DEFAULT = 5000
    MAX_PORTS_TRIED = 10

    def __init__(self) -> None:
        """Set the variables."""
        self.server_address = ""
        self.server_port = 0
        self.url_prefix: str = ""
        self.gui_port = 0
        self.log_level = 0
        self.token = ""
        self.scheme = ""
        self.workdir = ""
        self.allow_insecure_connections = False
        self.tls_certificate = ""
        self.tls_key = ""
        self.set_up()

    def set_up(self) -> None:
        """Reset variables."""
        self.determine_values_from_environment()

    def determine_values_from_environment(self) -> None:
        """Set variables based on environment values."""
        args = self.get_command_line_args()
        self.server_address = determine_ip()
        self.server_port = self.determine_server_port(args)
        self.url_prefix = self.determine_url_prefix(args)
        self.gui_port = self.determine_gui_port(args)
        self.log_level = self.determine_log_level(args)
        self.token = secrets.token_hex(16)
        self.allow_insecure_connections = args.get("allow_insecure_connections", False)
        self.tls_certificate = args.get("cert", "")
        self.tls_key = args.get("key", "")
        self.workdir = args.get("workdir_location", WORKDIR_LOCATION)
        self.scheme = "http" if self.allow_insecure_connections else "https"

    @property
    def global_config_directory(self) -> str:
        """Get the directory for global config files."""
        return os.path.join(
            os.environ.get("HOME", ""),
            ".neural_compressor",
        )

    def get_command_line_args(self) -> Dict:
        """Return arguments passed in command line."""
        parser = argparse.ArgumentParser(
            description="Run Neural Insights server.",
        )
        parser.add_argument(
            "-p",
            "--port",
            type=int,
            help="server port number to listen on",
        )
        parser.add_argument(
            "-P",
            "--gui-port",
            type=int,
            help="port number for GUI",
        )
        parser.add_argument(
            "-U",
            "--url-prefix",
            type=str,
            default="",
            help="URL prefix for Neural Insights instance.",
        )
        parser.add_argument(
            "--allow-insecure-connections",
            action="store_true",
            help="run server without encryption",
        )
        parser.add_argument(
            "--cert",
            type=str,
            default="",
            help="TLS Certificate to use",
        )
        parser.add_argument(
            "--key",
            type=str,
            default="",
            help="TLS private key to use",
        )
        parser.add_argument(
            "--workdir-location",
            type=str,
            default=WORKDIR_LOCATION,
            help="Path to work directory.",
        )
        parser.add_argument(
            "--verbose",
            "-v",
            action="count",
            default=2,
            help="verbosity of logging output, use -vv and -vvv for even more logs",
        )
        return vars(parser.parse_args())

    def determine_server_port(self, args: Dict) -> int:
        """Return port to be used by the server.

        Will raise a NotFoundException if port is already in use.

        When port given in command line, only that port will be tried.
        When no port specified will try self.MAX_PORTS_TRIED times,
        starting with self.PORT_DEFAULT.
        """
        command_line_port = args.get("port")
        if command_line_port is not None:
            self._ensure_valid_port(command_line_port)
            if self.is_port_taken(command_line_port):
                raise NotFoundException(
                    f"Port {command_line_port} already in use, exiting.",
                )
            else:
                return command_line_port

        ports = [self.PORT_DEFAULT] + randint(
            1025,
            65536,
            self.MAX_PORTS_TRIED - 1,
        ).tolist()

        for port in ports:
            if not self.is_port_taken(port):
                return port

        raise NotFoundException(
            f"Unable to find a free port in {len(ports)} attempts, exiting.",
        )

    def determine_gui_port(self, args: Dict) -> int:
        """Return port to be used by the GUI client.

        Will return self.server_port unless specified in configuration.
        """
        command_line_port = args.get("gui_port")
        if command_line_port is not None:
            self._ensure_valid_port(command_line_port)
            return command_line_port
        return self.server_port

    def is_port_taken(self, port: int) -> bool:
        """Return if given port is already in use."""
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        try:
            s.bind((self.server_address, port))
        except socket.error:
            return True
        finally:
            s.close()

        return False

    def determine_log_level(self, args: Dict) -> int:
        """Determine log level based on parameters given."""
        verbosity_mapping = [
            logging.CRITICAL,
            logging.WARNING,
            logging.INFO,
            logging.DEBUG,
        ]
        verbosity: int = args.get("verbose")  # type:ignore
        try:
            return verbosity_mapping[verbosity]
        except IndexError:
            return logging.DEBUG

    @staticmethod
    def determine_url_prefix(args: dict) -> str:
        """Determine url prefix based on parameters given."""
        url_prefix = args.get("url_prefix", "")
        if isinstance(url_prefix, str) and not url_prefix.startswith("/"):
            url_prefix = f"/{url_prefix}"
        return url_prefix

    def get_url(self) -> str:
        """Return URL to access application."""
        base_url = f"{self.scheme}://{self.server_address}:{self.gui_port}"
        if self.url_prefix != "/":
            base_url = f"{base_url}{self.url_prefix}"
        return f"{base_url}/?token={self.token}"

    def dump_token_to_file(self) -> None:
        """Dump token to file."""
        token_filepath = os.path.join(WORKDIR_LOCATION, "token")
        os.makedirs(os.path.dirname(token_filepath), exist_ok=True)

        if sys.platform == "win32":
            self.create_secured_token_file_win(token_filepath)

        try:
            token_file = os.open(token_filepath, flags=os.O_WRONLY | os.O_CREAT, mode=0o600)
            os.write(token_file, self.token.encode())
        except Exception as err:
            raise err
        finally:
            os.close(token_file)

        log.debug(f"Token has been dumped to {token_filepath}.")

    @staticmethod
    def create_secured_token_file_win(token_filepath: str):
        """Create secured file on Windows OS."""
        import ntsecuritycon as con  # pylint: disable=import-error
        import win32api  # pylint: disable=import-error
        import win32file  # pylint: disable=import-error
        import win32security  # pylint: disable=import-error

        username = win32api.GetUserName()
        os.makedirs(os.path.dirname(token_filepath), exist_ok=True)

        if os.path.exists(token_filepath):
            os.remove(token_filepath)

        security_descriptor = win32security.SECURITY_DESCRIPTOR()
        user_sid, _, _ = win32security.LookupAccountName("", username)

        access_rights = con.FILE_ALL_ACCESS

        dacl = win32security.ACL()
        dacl.AddAccessAllowedAce(win32security.ACL_REVISION, access_rights, user_sid)

        security_descriptor.SetSecurityDescriptorDacl(1, dacl, 0)

        security_attributes = win32security.SECURITY_ATTRIBUTES()
        security_attributes.SECURITY_DESCRIPTOR = security_descriptor

        handle = win32file.CreateFile(
            token_filepath,
            win32file.GENERIC_WRITE,
            win32file.FILE_SHARE_READ,
            security_attributes,
            win32file.CREATE_NEW,
            win32file.FILE_ATTRIBUTE_NORMAL,
            None,
        )

        win32file.CloseHandle(handle)

    def _ensure_valid_port(self, port: int) -> None:
        """Validate if proposed port number is allowed by TCP/IP."""
        if port < 1:
            raise ValueError(f"Lowest allowed port number is 1, attempted to use: {port}")
        if port > 65535:
            raise ValueError(f"Highest allowed port number is 65535, attempted to use: {port}")



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test Domain."""

import unittest

from neural_insights.components.model.domain import Domain


class TestDomain(unittest.TestCase):
    """Test Domain class."""

    def test_none_domain(self) -> None:
        """Test that unknown domain serializes as expected."""
        domain = Domain()

        expected = {
            "domain": "",
            "domain_flavour": "",
        }

        self.assertEqual(expected, domain.serialize())

    def test_domain_without_flavour(self) -> None:
        """Test that domain serializes as expected."""
        domain = Domain(domain="foo")

        expected = {
            "domain": "foo",
            "domain_flavour": "",
        }

        self.assertEqual(expected, domain.serialize())

    def test_domain_with_flavour(self) -> None:
        """Test that domain serializes as expected."""
        domain = Domain(domain="foo", domain_flavour="bar")

        expected = {
            "domain": "foo",
            "domain_flavour": "bar",
        }

        self.assertEqual(expected, domain.serialize())

    def test_domain_with_flavour_only_fails(self) -> None:
        """Test that domain can't be created given flavour only."""
        with self.assertRaisesRegex(ValueError, "Domain must be set when given flavour"):
            Domain(domain_flavour="bar")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test ModelRepository."""

import unittest

from neural_insights.components.model.repository import ModelRepository
from neural_insights.utils.consts import Frameworks
from neural_insights.utils.exceptions import NotFoundException


class TestModelRepository(unittest.TestCase):
    """Test ModelRepository class."""

    def test_onnx_is_model_path(self) -> None:
        """Test if onnx file is recognized correctly."""
        path = "/home/user/model.onnx"
        result = ModelRepository.is_model_path(path)
        self.assertTrue(result)

    def test_mp3_is_model_path(self) -> None:
        """Test if mp3 file is recognized correctly."""
        path = "/home/user/favourite_song.mp3"
        result = ModelRepository.is_model_path(path)
        self.assertFalse(result)

    def test_get_frameworks(self) -> None:
        """Test getting frameworks."""
        expected = [Frameworks.ONNX.value, Frameworks.PT.value, Frameworks.TF.value]

        repository = ModelRepository()
        actual = repository.get_frameworks()

        self.assertEqual(expected, actual)

    def test_framework_from_path_for_known_model(self) -> None:
        """Test get_framework_from_path."""
        actual = ModelRepository.get_framework_from_path("/home/user/model.onnx")
        self.assertEqual(Frameworks.ONNX.value, actual)

    def test_framework_from_path_for_unknown_model(self) -> None:
        """Test get_framework_from_path."""
        with self.assertRaises(NotFoundException):
            ModelRepository.get_framework_from_path("/home/user/favourite_song.mp3")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The diagnosis class."""
import os
import pickle
from abc import abstractmethod
from typing import Any, List, Optional

from neural_insights.components.diagnosis.op_details import OpDetails
from neural_insights.components.diagnosis.op_entry import OpEntry
from neural_insights.components.diagnosis.weights_details import WeightsDetails
from neural_insights.components.model.model import Model
from neural_insights.components.workload_manager.workload import Workload
from neural_insights.utils.exceptions import ClientErrorException, InternalException
from neural_insights.utils.logger import log
from neural_insights.utils.utils import check_module


class Diagnosis:
    """Diagnosis class."""

    def __init__(self, workload: Workload):
        """Initialize diagnosis."""
        self.workload_location = workload.workload_location
        self.model_path = workload.model_path

    @abstractmethod
    def model(self) -> Model:
        """Neural Insights model instance."""
        raise NotImplementedError

    def get_tensors_info(self, model_type: str = "optimized") -> dict:
        """Get information about tensors."""
        tensors_filenames = {
            "input": os.path.join("fp32", "inspect_result.pkl"),
            "optimized": os.path.join("quan", "inspect_result.pkl"),
        }

        tensors_filename = tensors_filenames.get(model_type, None)
        if tensors_filename is None:
            raise InternalException(f"Could not find tensors data for {model_type} model.")
        tensors_path = os.path.join(
            self.workload_location,
            "inspect_saved",
            tensors_filename,
        )
        if not os.path.exists(tensors_path):
            raise ClientErrorException("Could not find tensor data for specified optimization.")
        with open(tensors_path, "rb") as tensors_pickle:
            dump_tensor_result = pickle.load(tensors_pickle)
        return dump_tensor_result

    def load_quantization_config(self) -> dict:
        """Get config quantization data."""
        config_path = os.path.join(
            self.workload_location,
            "inspect_saved",
            "cfg.pkl",
        )
        if not os.path.exists(config_path):
            log.debug("Could not find config data for specified optimization. Getting data from inspect files.")
            input_model_tensors: dict = self.get_tensors_info(model_type="input")["activation"][0]
            optimized_model_tensors: dict = self.get_tensors_info(model_type="optimized")["activation"][0]
            common_ops = list(set(input_model_tensors.keys()) & set(optimized_model_tensors.keys()))
            config_data = {
                "op": {},
            }
            for op in common_ops:
                config_data["op"].update({(op,): {}})
            return config_data

        with open(config_path, "rb") as config_pickle:
            config_data = pickle.load(config_pickle)
        return config_data

    def get_op_list(self) -> List[dict]:
        """Get OP list for model."""
        check_module("numpy")
        import numpy as np

        op_list: List[dict] = []

        input_model_tensors: dict = self.get_tensors_info(model_type="input")["activation"][0]
        optimized_model_tensors: dict = self.get_tensors_info(model_type="optimized")["activation"][0]

        minmax_file_path = os.path.join(
            self.workload_location,
            "inspect_saved",
            "activation_min_max.pkl",
        )

        try:
            with open(minmax_file_path, "rb") as min_max_file:
                min_max_data: dict = pickle.load(min_max_file)
        except FileNotFoundError:
            log.debug("Could not find minmax file.")
            common_ops = list(set(input_model_tensors.keys()) & set(optimized_model_tensors.keys()))
            min_max_data = dict(zip(common_ops, [{"min": None, "max": None}] * len(common_ops)))

        for op_name, min_max in min_max_data.items():
            mse = self.calculate_mse(op_name, input_model_tensors, optimized_model_tensors)
            if mse is None or np.isnan(mse):
                continue
            min = min_max.get("min", None)
            max = min_max.get("max", None)

            if min is not None:
                min = float(min)
            if max is not None:
                max = float(max)

            op_entry = OpEntry(op_name, mse, min, max)
            op_list.append(op_entry.serialize())
        return op_list

    def get_weights_details(self, inspect_type: str) -> List[WeightsDetails]:
        """Get weights details for model."""
        weights_details = []

        minmax_file_path = os.path.join(
            self.workload_location,
            "inspect_saved",
            "activation_min_max.pkl",
        )
        with open(minmax_file_path, "rb") as min_max_file:
            min_max_data: dict = pickle.load(min_max_file)

        input_model_tensors: dict = self.get_tensors_info(model_type="input")[inspect_type]
        optimized_model_tensors: dict = self.get_tensors_info(model_type="optimized")[inspect_type]
        if inspect_type == "activation":
            input_model_tensors = input_model_tensors[0]
            optimized_model_tensors = optimized_model_tensors[0]
        common_ops = list(set(input_model_tensors.keys()) & set(optimized_model_tensors.keys()))
        for op_name in common_ops:
            input_model_op_tensors = input_model_tensors[op_name]
            optimized_model_op_tensors = optimized_model_tensors[op_name]

            if op_name not in min_max_data.keys():
                continue

            if isinstance(input_model_op_tensors, dict):
                for (input_op_name, input_op_values), (optimized_op_name, optimized_op_values) in zip(
                    input_model_op_tensors.items(), optimized_model_op_tensors.items()
                ):
                    if input_op_values.ndim != 4 or optimized_op_values.ndim != 4:
                        continue

                    weights_entry = WeightsDetails(
                        input_op_name,
                        input_op_values,
                        optimized_op_values,
                    )
                    weights_details.append(weights_entry)
        return weights_details

    def calculate_mse(
        self,
        op_name: str,
        input_model_tensors: dict,
        optimized_model_tensors: dict,
    ) -> Optional[float]:
        """Calculate MSE for specified tensors."""
        input_model_op_data = input_model_tensors.get(op_name, None)
        optimized_model_op_data = optimized_model_tensors.get(op_name, None)

        if input_model_op_data is None or optimized_model_op_data is None:
            return None

        mse: float = self.mse_metric_gap(
            next(iter(input_model_op_data.values()))[0],
            next(iter(optimized_model_op_data.values()))[0],
        )

        return mse

    def get_op_details(self, name: str) -> Optional[OpDetails]:
        """Get details of specific OP."""
        config_data = self.load_quantization_config()
        for op_tuple, op_details in config_data["op"].items():
            if op_tuple[0] == name:
                return OpDetails(name, op_details)
        return None

    def get_histogram_data(self, op_name: str, inspect_type: str) -> list:
        """Get data to draw histogram."""
        tensors = self.get_tensors_info(model_type="optimized").get(inspect_type, None)
        if tensors is None:
            raise ClientErrorException(
                f"Could not get tensor information for {inspect_type} type.",
            )

        if inspect_type == "activation":
            tensors = tensors[0]

        op_tensors: Optional[dict] = tensors.get(op_name, None)
        if op_tensors is None:
            raise ClientErrorException(
                f"Could not get tensor information for {op_name} OP.",
            )

        op_histograms = []
        for tensor_name, tensor_data_raw in op_tensors.items():
            tensor_histograms = []

            if tensor_data_raw.ndim == 1:
                tensor_data = [tensor_data_raw]
            elif tensor_data_raw.ndim == 2:
                tensor_data = tensor_data_raw[0]
            else:
                tensor_data = tensor_data_raw[0]
            for tensor_channel_data in tensor_data:
                tensor_histograms.append(
                    {
                        "data": tensor_channel_data.flatten().tolist(),
                    },
                )

            op_histograms.append(
                {
                    "name": f"{tensor_name} {inspect_type} histogram",
                    "histograms": tensor_histograms,
                },
            )

        return op_histograms

    @staticmethod
    def mse_metric_gap(fp32_tensor: Any, dequantize_tensor: Any) -> float:
        """Calculate the euclidean distance between fp32 tensor and int8 dequantize tensor.

        Args:
            fp32_tensor (tensor): The FP32 tensor.
            dequantize_tensor (tensor): The INT8 dequantize tensor.
        """
        check_module("numpy")
        import numpy as np

        # Get input tensor min max
        fp32_max = np.max(fp32_tensor)  # type: ignore
        fp32_min = np.min(fp32_tensor)  # type: ignore

        # Get input dequantize tensor min max
        dequantize_max = np.max(dequantize_tensor)  # type: ignore
        dequantize_min = np.min(dequantize_tensor)  # type: ignore

        # Normalize tensor values
        fp32_tensor = (fp32_tensor - fp32_min) / (fp32_max - fp32_min)
        dequantize_tensor = (dequantize_tensor - dequantize_min) / (dequantize_max - dequantize_min)

        diff_tensor = fp32_tensor - dequantize_tensor
        euclidean_dist = np.sum(diff_tensor**2)  # type: ignore
        return euclidean_dist / fp32_tensor.size

    def get_weights_data(self, op_name: str, channel_normalization=True) -> list:
        """Get weights data for optimized model."""
        from PIL import Image

        check_module("numpy")
        import numpy as np

        tensors = self.get_tensors_info(model_type="optimized").get("weight", None)
        if tensors is None:
            raise ClientErrorException(
                "Could not get tensor information to display activations.",
            )

        op_tensors: Optional[dict] = tensors.get(op_name, None)
        if op_tensors is None:
            raise ClientErrorException(
                f"Could not get tensor information for {op_name} OP.",
            )

        weights = []
        for tensor_name, tensor_data_raw in op_tensors.items():
            if tensor_data_raw.ndim != 4:
                continue
            tensor_data = tensor_data_raw[0]
            shapes_order = self.model.shape_elements_order  # pylint: disable=no-member
            channels_index = shapes_order.index("channels")
            new_order = [channels_index]
            new_order.extend([x for x in range(len(shapes_order)) if x != channels_index])

            tensor_data = np.transpose(tensor_data, new_order)

            if tensor_data.shape[1] != tensor_data.shape[2]:
                continue

            for tensor in tensor_data:
                if channel_normalization:
                    tensor = 255 * (tensor - np.min(tensor)) / (np.max(tensor) - np.min(tensor))
                img = Image.fromarray(tensor)
                img = img.convert("L")
                img.show()
                weights.append(tensor.tolist())
        return weights



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Request data processor."""

from typing import Any, Dict

from neural_insights.utils.exceptions import ClientErrorException


class RequestDataProcessor:
    """Request data processor."""

    @staticmethod
    def get_string_value(data: Dict[str, Any], name: str) -> str:
        """Get string value from request."""
        try:
            return data[name][0]
        except KeyError:
            raise ClientErrorException(f"Missing {name} parameter")



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Domain Exceptions."""


class NotFoundException(Exception):
    """Not Found Exception."""

    pass


class AccessDeniedException(Exception):
    """Access Denied."""

    pass


class ClientErrorException(Exception):
    """Generic user error."""

    pass


class InternalException(Exception):
    """Internal application exception."""

    pass



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test Attribute."""

import unittest
import uuid

from neural_insights.components.graph.attribute import Attribute
from neural_insights.utils.json_serializer import JsonSerializer


def _get_random_string() -> str:
    """Create random string to be used."""
    return uuid.uuid4().hex


class TestAttribute(unittest.TestCase):
    """Test Attribute class."""

    def test_setting_parameters(self) -> None:
        """Test if parameters are correctly set."""
        name = _get_random_string()
        attribute_type = _get_random_string()
        value = _get_random_string()

        attribute = Attribute(name=name, attribute_type=attribute_type, value=value)

        self.assertEqual(name, attribute.name)
        self.assertEqual(attribute_type, attribute.attribute_type)
        self.assertEqual(value, attribute.value)

    def test_serialization(self) -> None:
        """Test if Attribute is serialized as expected."""
        name = _get_random_string()
        attribute_type = _get_random_string()
        value = _get_random_string()

        attribute = Attribute(name=name, attribute_type=attribute_type, value=value)

        expected_serialized_object = {
            "name": name,
            "attribute_type": attribute_type,
            "value": value,
        }
        serialized = JsonSerializer.serialize_item(attribute)
        self.assertEqual(expected_serialized_object, serialized)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The workload module for Neural Insights workloads."""

import os
from datetime import datetime
from typing import Any, Dict, Optional
from uuid import uuid4

from neural_insights.utils.consts import Frameworks, WorkloadModes, WorkloadStatus
from neural_insights.utils.exceptions import InternalException
from neural_insights.utils.json_serializer import JsonSerializer
from neural_insights.utils.utils import get_framework_from_path


class Workload(JsonSerializer):
    """Workload class."""

    def __init__(self, data: Optional[dict] = None):
        """Initialize Workload."""
        super().__init__()
        if data is None:
            data = {}
        self.uuid = str(data.get("uuid", uuid4()))
        self.creation_time: int = int(data.get("creation_time", datetime.now().timestamp()))
        self.workload_location: str = data.get("workload_location", None)

        mode = data.get("mode")
        self.workload_name = data.get("workload_name", mode)
        if not isinstance(mode, WorkloadModes) and isinstance(mode, str):
            mode = WorkloadModes(mode)
        self.mode: WorkloadModes = mode

        status = data.get("status", WorkloadStatus.NEW)
        if not isinstance(status, WorkloadStatus) and isinstance(status, str):
            status = WorkloadStatus(status)
        self.status: WorkloadStatus = status

        self._model_path: str = data.get("model_path", None)

        framework = data.get("framework", None)
        if not isinstance(framework, Frameworks):
            if framework is None and self.model_path is not None:
                framework = get_framework_from_path(self.model_path)
            if isinstance(framework, str):
                framework = Frameworks(framework)
        self.framework: Optional[Frameworks] = framework

        self.model_summary_file = data.get("model_summary_file", None)

    @property
    def model_path(self) -> str:
        """Get model_path."""
        return self._model_path

    @model_path.setter
    def model_path(self, model_path: str) -> None:
        if not os.path.exists(model_path):
            raise InternalException("Could not locate model.")
        self._model_path = model_path
        self.framework = Frameworks(get_framework_from_path(self.model_path))

    def serialize(self, serialization_type: str = "default") -> Dict[str, Any]:
        """Serialize Workload class."""
        return {
            "uuid": self.uuid,
            "workload_name": self.workload_name,
            "framework": self.framework.value,
            "workload_location": self.workload_location,
            "mode": self.mode.value,
            "model_path": self.model_path,
            "model_summary_file": self.model_summary_file,
            "status": self.status.value,
            "creation_time": self.creation_time,
        }



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test Graph classes."""

import unittest
import uuid
from typing import List

from neural_insights.components.graph.graph import Graph
from neural_insights.components.graph.node import Node


def _get_random_string() -> str:
    """Create random string to be used."""
    return uuid.uuid4().hex


def _get_random_node() -> Node:
    """Create a Node with random values."""
    return Node(id=_get_random_string(), label=_get_random_string())


def _get_fake_graph() -> Graph:
    """Create graph with fake data."""
    node_1 = Node(id="node_1", label="Input")
    node_2 = Node(id="node_2", label="SomeNode")
    node_3 = Node(id="node_3", label="SomeNode")
    node_4 = Node(id="node_4", label="SomeNode")
    node_5 = Node(id="node_5", label="Softmax")

    graph = Graph()
    graph.add_node(node_1)
    graph.add_node(node_2)
    graph.add_node(node_3)
    graph.add_node(node_4)
    graph.add_node(node_5)

    graph.add_edge(source_id=node_1.id, target_id=node_2.id)
    graph.add_edge(source_id=node_1.id, target_id=node_3.id)
    graph.add_edge(source_id=node_1.id, target_id=node_4.id)
    graph.add_edge(source_id=node_2.id, target_id=node_5.id)
    graph.add_edge(source_id=node_3.id, target_id=node_5.id)
    graph.add_edge(source_id=node_4.id, target_id=node_5.id)

    return graph


def _get_fake_sequential_graph() -> Graph:
    """Create graph with fake data."""
    node_1 = Node(id="node_1", label="Input")
    node_2 = Node(id="node_2", label="Conv2D")
    node_3 = Node(id="node_3", label="AddBias")
    node_4 = Node(id="node_4", label="ReLU")
    node_5 = Node(id="node_5", label="Softmax")

    graph = Graph()
    graph.add_node(node_1)
    graph.add_node(node_2)
    graph.add_node(node_3)
    graph.add_node(node_4)
    graph.add_node(node_5)

    graph.add_edge(source_id=node_1.id, target_id=node_2.id)
    graph.add_edge(source_id=node_2.id, target_id=node_3.id)
    graph.add_edge(source_id=node_3.id, target_id=node_4.id)
    graph.add_edge(source_id=node_4.id, target_id=node_5.id)

    return graph


class TestGraph(unittest.TestCase):
    """Test Graph class."""

    def test_empty_graph_has_no_nodes(self) -> None:
        """Test if empty graph has no nodes."""
        graph = Graph()
        self.assertEqual([], graph.nodes)

    def test_empty_graph_has_no_edges(self) -> None:
        """Test if empty graph has no edges."""
        graph = Graph()
        self.assertEqual([], graph.edges)

    def test_graph_with_one_node_returns_only_one_node(self) -> None:
        """Test if empty graph with one node returns it correctly."""
        node = _get_random_node()
        graph = Graph()
        graph.add_node(node)

        self.assertEqual([node], graph.nodes)

    def test_adding_edge_when_no_nodes_does_not_add_an_adge(self) -> None:
        """Test if adding an edge to empty graph fails as expected."""
        source_id = _get_random_string()
        target_id = _get_random_string()

        graph = Graph()
        result = graph.add_edge(source_id=source_id, target_id=target_id)

        self.assertEqual(False, result)
        self.assertEqual([], graph.edges)

    def test_adding_edge_between_nodes_works(self) -> None:
        """Test adding an edge."""
        source = _get_random_node()
        target = _get_random_node()

        graph = Graph()
        graph.add_node(source)
        graph.add_node(target)
        result = graph.add_edge(source_id=source.id, target_id=target.id)

        self.assertEqual(True, result)

        edges = graph.edges
        self.assertEqual(1, len(edges))

        edge = edges[0]
        self.assertEqual(source, edge._source)
        self.assertEqual(target, edge._target)

    def test_comparing_to_something_else_fails(self) -> None:
        """Test if comparing Graph to something other fails."""
        with self.assertRaises(NotImplementedError):
            Graph() == 1

    def test_highlight_pattern(self) -> None:
        """Test highlight_pattern."""
        graph = _get_fake_sequential_graph()
        op_name = "node_2"
        pattern = ["Conv2D", "AddBias", "ReLU"]

        graph.highlight_pattern(op_name, pattern)

        self.assertFalse(graph.get_node("node_1").highlight)
        self.assertTrue(graph.get_node("node_2").highlight)
        self.assertTrue(graph.get_node("node_3").highlight)
        self.assertTrue(graph.get_node("node_4").highlight)
        self.assertFalse(graph.get_node("node_5").highlight)

    def test_get_multiple_target_nodes(self) -> None:
        """Test getting multiple target nodes using get_target_nodes."""
        graph = _get_fake_graph()
        result = [node.id for node in graph.get_target_nodes("node_1")]

        expected = ["node_2", "node_3", "node_4"]

        self.assertListEqual(result, expected)

    def test_get_single_target_node(self) -> None:
        """Test getting single target node using get_target_nodes."""
        graph = _get_fake_graph()
        result = [node.id for node in graph.get_target_nodes("node_3")]

        expected = ["node_5"]

        self.assertListEqual(result, expected)

    def test_get_target_node_for_last_node(self) -> None:
        """Test getting target node for last node using get_target_nodes."""
        graph = _get_fake_graph()
        result = [node.id for node in graph.get_target_nodes("node_5")]

        expected: List[str] = []

        self.assertListEqual(result, expected)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test Edge."""

import unittest
import uuid

from neural_insights.components.graph.edge import Edge
from neural_insights.components.graph.node import Node
from neural_insights.utils.json_serializer import JsonSerializer


def _get_random_string() -> str:
    """Create random string to be used."""
    return uuid.uuid4().hex


def _get_random_node() -> Node:
    """Create a Node with random values."""
    return Node(id=_get_random_string(), label=_get_random_string())


class TestEdge(unittest.TestCase):
    """Test Edge class."""

    def test_setting_parameters(self) -> None:
        """Test if parameters are correctly set."""
        source_node = _get_random_node()
        target_node = _get_random_node()

        edge = Edge(
            source=source_node,
            target=target_node,
        )

        self.assertEqual(source_node, edge._source)
        self.assertEqual(source_node.id, edge.source)
        self.assertEqual(target_node, edge._target)
        self.assertEqual(target_node.id, edge.target)

    def test_serialization(self) -> None:
        """Test if Edge is serialized as expected."""
        source_node = _get_random_node()
        target_node = _get_random_node()

        edge = Edge(
            source=source_node,
            target=target_node,
        )

        expected_serialized_object = {
            "source": source_node.id,
            "target": target_node.id,
        }
        serialized = JsonSerializer.serialize_item(edge)
        self.assertEqual(expected_serialized_object, serialized)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Graph class."""

from typing import Dict, List

from neural_insights.components.graph.edge import Edge
from neural_insights.components.graph.node import Node
from neural_insights.utils.exceptions import NotFoundException
from neural_insights.utils.json_serializer import JsonSerializer
from neural_insights.utils.logger import log


class Graph(JsonSerializer):
    """Model graph definition."""

    def __init__(self) -> None:
        """Construct object."""
        super().__init__()
        self._nodes: Dict[str, Node] = {}
        self._edges: List[Edge] = []

    def add_node(self, node: Node) -> None:
        """Add a Node to graph."""
        self._nodes[node.id] = node

    @property
    def nodes(self) -> List[Node]:
        """Return nodes as a list."""
        return list(self._nodes.values())

    @property
    def edges(self) -> List[Edge]:
        """Return edges as a list."""
        return self._edges

    def add_edge(self, source_id: str, target_id: str) -> bool:
        """Add an Edge to graph."""
        try:
            source = self.get_node(source_id)
            target = self.get_node(target_id)
        except NotFoundException as err:
            log.debug(
                f"Got an error: {str(err)} while attempted " f"to add an Edge from {source_id} to {target_id}",
            )
            return False
        self._edges.append(Edge(source, target))
        return True

    def get_node(self, id: str) -> Node:
        """Get a node by id."""
        if id not in self._nodes:
            raise NotFoundException(f"Node id: {id} not found in Graph")
        return self._nodes[id]

    def highlight_pattern(self, op_name: str, pattern: List[str]) -> None:
        """Highlight pattern in graph."""
        source_op = op_name
        self.get_node(source_op).highlight = True
        for op in pattern[1:]:
            target_nodes = self.get_target_nodes(source_op)
            for target_node in target_nodes:
                if target_node.label == op:
                    self.get_node(target_node.id).highlight = True
                    source_op = target_node.id
                    continue

    def get_target_nodes(self, op_name: str) -> List[Node]:
        """Get target nodes from specified op."""
        target_nodes: List[Node] = []

        for edge in self.edges:
            if edge.source == op_name:
                target_nodes.append(self.get_node(edge.target))

        return target_nodes



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Response generator."""

from werkzeug.wrappers import Response

from neural_insights.utils.exceptions import (
    AccessDeniedException,
    ClientErrorException,
    InternalException,
    NotFoundException,
)


class ResponseGenerator:
    """Response generator class."""

    @staticmethod
    def add_refresh(response: Response, refresh_time: int) -> Response:
        """Add Refresh header to response."""
        response.headers["refresh"] = refresh_time
        return response

    @staticmethod
    def from_exception(exception: Exception) -> Response:
        """Create Response from Exception."""
        return Response(
            response=str(exception),
            status=ResponseGenerator.get_status_code_for_exception(exception),
        )

    @staticmethod
    def get_status_code_for_exception(exception: Exception) -> int:
        """Get HTTP status code for Exception."""
        if isinstance(exception, ClientErrorException):
            return 400
        if isinstance(exception, AccessDeniedException):
            return 403
        if isinstance(exception, NotFoundException):
            return 404
        if isinstance(exception, InternalException):
            return 500
        return 500



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The OpDetails class."""
from typing import Any, Dict, List

from neural_insights.utils.json_serializer import JsonSerializer


class OpDetails(JsonSerializer):
    """OP details class."""

    def __init__(self, op_name: str, op_details: dict) -> None:
        """Initialize OP details."""
        super().__init__()
        self.op_name: str = op_name
        self.pattern: OpPattern = OpPattern(op_details.get("pattern", {}))
        self.weight: OpWeights = OpWeights(op_details.get("weight", {}))
        self.activation: OpActivation = OpActivation(op_details.get("activation", {}))

    def serialize(
        self,
        serialization_type: str = "default",
    ) -> Dict[str, Any]:
        """Serialize OpDetails."""
        return {
            "OP name": self.op_name,
            "Pattern": self.pattern.serialize(),
            "Weights": self.weight.serialize(),
            "Activation": self.activation.serialize(),
        }


class OpPattern(JsonSerializer):
    """OP pattern class."""

    def __init__(self, pattern_data: dict):
        """Initialize OP pattern."""
        super().__init__()
        self.sequence: List[str] = pattern_data.get("sequence", "").split(",")
        self.precision: str = pattern_data.get("precision", None)


class OpWeights(JsonSerializer):
    """OP weights class."""

    def __init__(self, weights_data: dict):
        """Initialize OP weights."""
        super().__init__()
        self.dtype: str = weights_data.get("dtype", None)
        self.granularity: str = weights_data.get("granularity", None)


class OpActivation(JsonSerializer):
    """OP activation class."""

    def __init__(self, op_activation: dict):
        """Initialize OP activation."""
        super().__init__()
        self.dtype: str = op_activation.get("dtype", None)



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils test."""

import unittest
from unittest.mock import MagicMock, patch

from neural_insights.utils.consts import Frameworks
from neural_insights.utils.exceptions import (
    ClientErrorException,
)
from neural_insights.utils.utils import (
    check_module,
    get_file_extension,
    get_framework_from_path,
)

fake_metrics: dict = {
    "topk": {},
    "COCOmAP": {},
    "MSE": {},
    "RMSE": {},
    "MAE": {},
    "metric1": {},
}


class TestUtils(unittest.TestCase):
    """Value parser tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Parser tests constructor."""
        super().__init__(*args, **kwargs)

    @patch("neural_insights.components.model.tensorflow.frozen_pb.get_model_type")
    def test_get_tensorflow_framework_from_path(self, mocked_get_model_type: MagicMock) -> None:
        """Test getting framework name from path."""
        mocked_get_model_type.return_value = "frozen_pb"
        path = "/home/user/model.pb"
        result = get_framework_from_path(path)
        self.assertEqual(result, Frameworks.TF.value)
        mocked_get_model_type.assert_called_with(path)

    def test_get_onnx_framework_from_path(self) -> None:
        """Test getting framework name from path."""
        path = "/home/user/model.onnx"
        result = get_framework_from_path(path)
        self.assertEqual(result, Frameworks.ONNX.value)

    def test_get_unknown_framework_from_path(self) -> None:
        """Test getting framework name from path."""
        path = "/home/user/model.some_extension"
        result = get_framework_from_path(path)
        self.assertIsNone(result)

    def test_get_file_extension(self) -> None:
        """Test getting file extension from path."""
        path = "/home/user/file.ext"
        result = get_file_extension(path)
        self.assertEqual(result, "ext")

    def test_get_file_with_dots_extension(self) -> None:
        """Test getting file extension from path."""
        path = "/home/user/file.name.ext2"
        result = get_file_extension(path)
        self.assertEqual(result, "ext2")

    def test_get_file_without_extension(self) -> None:
        """Test getting file extension from path."""
        path = "/home/user/file"
        result = get_file_extension(path)
        self.assertEqual(result, "")

    def test_check_module(self) -> None:
        """Test checking existing module."""
        check_module("os")

    def test_check_non_existing_module(self) -> None:
        """Test checking non existing module."""
        with self.assertRaises(ClientErrorException):
            check_module("non_existing_module")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Logger test."""

import logging
import unittest

from neural_insights.utils.logger import change_log_level, log


class TestLogger(unittest.TestCase):
    """Logger tests."""

    def test_changing_log_level(self) -> None:
        """Test default values."""
        change_log_level(logging.INFO)
        self.assertEqual(logging.INFO, log.level)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Environment manager class."""
import os
import sys


class Environment:
    """Environment manager class."""

    @staticmethod
    def ensure_workdir_exists_and_writeable() -> None:
        """Ensure that configured directory exists and can be used."""
        from neural_insights.utils.logger import log
        from neural_insights.web.configuration import Configuration

        configuration = Configuration()
        workdir = configuration.workdir
        error_message_tail = "Please ensure it is a directory that can be written to.\nExiting.\n"
        try:
            os.makedirs(workdir, exist_ok=True)
        except Exception as e:
            log.error(f"Unable to create workdir at {workdir}: {e}.\n{error_message_tail}")
            log.error(e)
            sys.exit(1)
        if not os.access(workdir, os.W_OK):
            log.error(f"Unable to create files at {workdir}.\n{error_message_tail}")
            sys.exit(2)



# -*- coding: utf-8 -*-
# Copyright (c) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""ExpiringDict test."""
import time
import unittest

from neural_insights.utils.expiring_dict import ExpiringDict, ExpiringDictItem


class TestExpiringDictItem(unittest.TestCase):
    """ExpiringDictItem tests."""

    def test_it_sets_values(self) -> None:
        """Test if values are set correctly."""
        value = "this is expected value"
        expires_at = time.time() + 2

        item = ExpiringDictItem(value=value, expires_at=expires_at)

        self.assertEqual(value, item.value)
        self.assertEqual(expires_at, item.expires_at)

    def test_it_expires_after_time_passed(self) -> None:
        """Test if ExpiringDictItem expires after given time."""
        value = "this is expected value"
        expires_at = time.time() + 2

        item = ExpiringDictItem(value=value, expires_at=expires_at)

        self.assertFalse(item.is_expired())

        time.sleep(3)
        self.assertTrue(item.is_expired())


class TestExpiringDict(unittest.TestCase):
    """ExpiringDict tests."""

    def test_items_disappear_after_expiration(self) -> None:
        """Test that item is not accessible after expiration."""
        key = "tested key"
        value = "expected value"

        items = ExpiringDict(ttl=2)

        with self.assertRaisesRegex(KeyError, expected_regex=key):
            items[key]

        items[key] = value

        self.assertEqual(value, items[key])

        time.sleep(3)

        with self.assertRaisesRegex(KeyError, expected_regex=key):
            items[key]

    def test_items_disappear_after_expiration_when_passed_in_constructor(self) -> None:
        """Test that item is not accessible after expiration."""
        key = "tested key"
        value = "expected value"

        items = ExpiringDict(
            initial_value={
                key: value,
            },
            ttl=2,
        )

        self.assertEqual(value, items[key])

        time.sleep(3)

        with self.assertRaisesRegex(KeyError, expected_regex=key):
            items[key]


if __name__ == "__main__":
    unittest.main()
